"use strict";

function _getDBConnection(config) {
    try {
        var collectionName = config.adapter.collection;
        if (!collectionName) return null;
        return scule.factoryCollection("scule+titanium://" + collectionName, {
            remoteBackup: config.adapter.remoteBackup
        });
    } catch (e) {
        LOGGER.error("[NoSQL Storage] - failed to get db connection");
        return null;
    }
}

function _capCollection(db, config) {
    if (!config.adapter.hasOwnProperty("cappedSize") || config.adapter.cappedSize < 1) return;
    var cappedSize = config.adapter.cappedSize;
    try {
        db.count({}, {}, function(count) {
            if (cappedSize >= count) return;
            db.remove({}, {
                $limit: count - cappedSize
            });
            db.commit();
        });
    } catch (e) {
        LOGGER.error("[NoSQL Storage] - failed to cap collection");
    }
}

function _upsertCollection(db, collection, options) {
    function insertCollection() {
        LOGGER.debug("[NoSQL Storage] - start upserting collection...");
        try {
            var collectionJSON = collection.toJSON(), idAttribute = collection.config.idAttribute || "id";
            for (var i = 0; i < collectionJSON.length; i++) {
                var modelJSON = collectionJSON[i];
                !modelJSON.hasOwnProperty("_id") && modelJSON.hasOwnProperty(idAttribute) && (modelJSON._id = modelJSON[idAttribute]);
                LOGGER.debug("[NoSQL Storage] - upserting model with id: " + modelJSON._id);
                db.save(modelJSON);
            }
            db.commit();
            deferred.resolve(collectionJSON);
        } catch (e) {
            LOGGER.error(e);
            deferred.reject(SyncError.createErrorResponse(500, "persistence_error", "failed to save data locally"));
        }
        _capCollection(db, collection.config);
    }
    var deferred = Q.defer();
    var merge = options.hasOwnProperty("merge") ? options.merge : collection.config.merge;
    if (merge) insertCollection(); else {
        LOGGER.debug("[NoSQL Storage] - delete previous storage before storing new collection");
        db.clear(insertCollection);
    }
    return deferred.promise;
}

function _upsertModel(db, model, options) {
    function insertModel() {
        LOGGER.debug("[NoSQL Storage] - upserting model with id: " + modelJSON._id);
        try {
            db.save(modelJSON);
            db.commit();
            deferred.resolve(modelJSON);
        } catch (e) {
            LOGGER.error(e);
            deferred.reject(SyncError.createErrorResponse(500, "persistence_error", "failed to save data locally"));
        }
        _capCollection(db, model.config);
    }
    var deferred = Q.defer(), modelJSON = model.toJSON(), idAttribute = model.config.idAttribute || "id";
    !modelJSON.hasOwnProperty("_id") && modelJSON.hasOwnProperty(idAttribute) && (modelJSON._id = modelJSON[idAttribute]);
    var merge = options.hasOwnProperty("merge") ? options.merge : model.config.merge;
    if (merge) insertModel(); else {
        LOGGER.debug("[NoSQL Storage] - delete previous stored model before storing new model");
        var query = modelJSON._id ? {
            _id: modelJSON._id
        } : {};
        db.remove(query, null, insertModel);
    }
    return deferred.promise;
}

function _fetchCollection(db, collection, options) {
    var deferred = Q.defer();
    try {
        options.conditions = options.conditions || {};
        options.conditions.$unset = {
            _id: true
        };
        if (options.hasOwnProperty("query")) {
            LOGGER.debug("[NoSQL Storage] - search collection by query");
            db.find(options.query, options.conditions, function(results) {
                deferred.resolve(results);
            });
        } else {
            LOGGER.debug("[NoSQL Storage] - find all");
            db.find({}, options.conditions, function(results) {
                deferred.resolve(results);
            });
        }
    } catch (e) {
        LOGGER.error(e);
        deferred.reject(SyncError.createErrorResponse(500, "query_error", "failed to query local data"));
    }
    return deferred.promise;
}

function _fetchModel(db, model, options) {
    var deferred = Q.defer();
    try {
        options.conditions = options.conditions || {};
        options.conditions.$unset = {
            _id: true
        };
        if (options.hasOwnProperty("query")) {
            LOGGER.debug("[NoSQL Storage] - find model by query, return the first match");
            db.find(options.query, options.conditions, function(results) {
                var result = results && results.length > 0 ? results[0] : null;
                deferred.resolve(result);
            });
        } else {
            var idAttribute = model.config.idAttribute || "id", id = model.get(idAttribute);
            LOGGER.debug("[NoSQL Storage] - find model by id: " + id);
            db.findOne(id, function(result) {
                result = _.clone(result);
                result && delete result._id;
                deferred.resolve(result);
            });
        }
    } catch (e) {
        LOGGER.error(e);
        deferred.reject(SyncError.createErrorResponse(500, "query_error", "failed to query local data"));
    }
    return deferred.promise;
}

function _removeCollection(db, collection, options) {
    var deferred = Q.defer();
    try {
        if (options.hasOwnProperty("query")) {
            LOGGER.debug("[NoSQL Storage] - remove collection by query");
            db.remove(options.query, options.conditions, function() {
                deferred.resolve();
            });
        } else {
            LOGGER.debug("[NoSQL Storage] - clear collection");
            db.clear(function() {
                deferred.resolve();
            });
        }
        db.commit();
    } catch (e) {
        LOGGER.error(e);
        deferred.reject(SyncError.createErrorResponse(500, "query_error", "failed to remove local data"));
    }
    return deferred.promise;
}

function _removeModel(db, model, options) {
    var deferred = Q.defer(), query = options.query;
    if (!query) {
        var idAttribute = model.config.idAttribute || "id", id = model.get(idAttribute);
        query = {
            _id: id
        };
    }
    try {
        db.remove(query, options.conditions, function() {
            deferred.resolve();
        });
        db.commit();
    } catch (e) {
        LOGGER.error(e);
        deferred.reject(SyncError.createErrorResponse(500, "query_error", "failed to remove local data"));
    }
    return deferred.promise;
}

var scule = require("/vendor/scule"), Q = require("vendor/q"), SyncError = require("/alloy/utils/sync_error");

module.exports = {
    fetch: function(model, options) {
        var db = _getDBConnection(model.config);
        if (!db) return SyncError.createErrorPromise(500, "db_conn_fail", "unable to get db connection.");
        if (model.hasOwnProperty("models")) {
            LOGGER.debug("[NoSQL Storage] - fetch collection");
            return _fetchCollection(db, model, options);
        }
        LOGGER.debug("[NoSQL Storage] - fetch model");
        return _fetchModel(db, model, options);
    },
    save: function(model, options) {
        var db = _getDBConnection(model.config);
        if (!db) return SyncError.createErrorPromise(500, "db_conn_fail", "unable to get db connection.");
        if (model.hasOwnProperty("models")) {
            LOGGER.debug("[NoSQL Storage] - save collection");
            return _upsertCollection(db, model, options);
        }
        LOGGER.debug("[NoSQL Storage] - save model");
        return _upsertModel(db, model, options);
    },
    remove: function(model, options) {
        var db = _getDBConnection(model.config);
        if (!db) return SyncError.createErrorPromise(500, "db_conn_fail", "unable to get db connection.");
        if (model.hasOwnProperty("models")) {
            LOGGER.debug("[NoSQL Storage] - remove collection");
            return _removeCollection(db, model, options);
        }
        LOGGER.debug("[NoSQL Storage] - remove model");
        return _removeModel(db, model, options);
    }
};