"use strict";

var Q = require("vendor/q"), utils = require("/utils/utils"), Uri = require("/vendor/uri");

var DEFAULT_TIME_OUT = 6e4, QOS_TIME_OUT = 1e4;

var SyncError = require("/alloy/utils/sync_error");

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

var _METHOD_MAP = {
    read: "GET",
    create: "POST",
    update: "PUT",
    "delete": "DELETE"
};

var _headers = {
    "Content-Type": "application/json;charset=UTF-8",
    Accept: "application/json",
    "User-Agent": Ti.userAgent
};

var parseSuccessResponse = function(client) {
    try {
        var json = JSON.parse(client.responseText);
        if ("object" == typeof json) return json;
    } catch (e) {
        LOGGER.info("reponse is not in JSON format");
    }
    return {
        response: client.responseText
    } || {};
};

var parseErrorResponse = function(client, response) {
    var json = {};
    try {
        json = JSON.parse(client.responseText);
    } catch (e) {
        LOGGER.info("[REST Adapter] Reponse is not in JSON format: " + client.responseText);
    }
    var error = {
        url: client.url,
        status: client.status,
        errorCode: json ? json.errorCode || json.error || response.error : "",
        errorDetails: json ? json.errorDetails : {},
        description: json ? json.description || json.error_description || response.error : ""
    };
    LOGGER.error("[REST Adapter] Returning error: " + JSON.stringify(error));
    return error;
};

var _lastMonitorId = Date.now();

var _lastValue = true;

var Monitor = function() {
    return {
        id: Date.now(),
        start: function() {
            var self = this;
            self.timer = setTimeout(function() {
                self.stop(false);
            }, QOS_TIME_OUT);
        },
        stop: function(flag) {
            var self = this;
            if (self.timer) {
                clearTimeout(self.timer);
                self.timer = null;
                if (void 0 !== flag && null !== flag && self.id >= _lastMonitorId) {
                    _lastMonitorId = self.id;
                    if (_lastValue !== flag) {
                        _lastValue = flag;
                        Ti.App.fireEvent("carma:network.experience", {
                            value: flag
                        });
                    }
                }
            }
        }
    };
};

var _connect = function(deferred, model, options) {
    if (!Ti.Network.online) {
        LOGGER.error("[Sync Adapter] - offline, unable to make HTTP request");
        return SyncError.createErrorPromise(522, "offline", "no network connectivity");
    }
    var client = Ti.Network.createHTTPClient({
        cache: options.cache || false,
        enableKeepAlive: options.enableKeepAlive || true,
        timeout: options.timeout || DEFAULT_TIME_OUT
    });
    var monitorable = true !== model.config.background && "true" !== options.background;
    var monitor = monitorable ? Monitor() : null;
    monitor && monitor.start();
    client.onload = function() {
        monitor && monitor.stop(true);
        options.success.call(this, {
            status: client.status,
            data: parseSuccessResponse(client)
        });
    };
    client.onerror = function(response) {
        if (monitor) {
            LOGGER.debug("[REST Adapter] - Experiencing bad internet connection... detected by the failure");
            var badConnection = response && response.code && ("1" === response.code.toString() || "2" === response.code.toString());
            monitor.stop(badConnection ? false : null);
        }
        var normalizedError = parseErrorResponse(client, response);
        if ("invalid_token" === normalizedError.errorCode) {
            Ti.App.fireEvent("carma:logout");
            return;
        }
        options.error.call(this, normalizedError);
    };
    if ("GET" === options.method && options.data) {
        var url = new Uri(options.url);
        for (var key in options.data) url.addQueryParam(key, options.data[key]);
        options.url = url.toString();
        delete options.data;
    }
    LOGGER.debug("[REST Adapter]: " + options.method + " " + options.url);
    client.open(options.method, options.url);
    LoggingService.log("network", options.url);
    for (var header in options.headers) {
        client.setRequestHeader(header, options.headers[header]);
        LOGGER.debug("[REST Adapter]: HEADER[" + header + "] = " + options.headers[header]);
    }
    options.beforeSend && options.beforeSend(client);
    client.send(options.data || null);
    LOGGER.debug("[REST Adapter] - post data: " + options.data);
    return deferred.promise;
};

var _wrapSuccess = function(deferred, model, options) {
    var successCallback = options.success;
    options.success = function(response) {
        successCallback && successCallback(response.data, response.status);
        deferred.resolve(model.toJSON());
    };
};

var _wrapError = function(deferred, model, options) {
    var errorCallback = options.error;
    options.error = function(response) {
        errorCallback && errorCallback(model, response, options);
        deferred.reject(response);
        LOGGER.error("[REST Adapter] - Error: " + JSON.stringify(response));
    };
};

var _wrapAPICall = function(deferred, model, options) {
    _wrapSuccess(deferred, model, options);
    _wrapError(deferred, model, options);
    return _connect(deferred, model, options);
};

var _read = function(deferred, model, options) {
    model.trigger("prefetch");
    return _wrapAPICall(deferred, model, options);
};

var _save = function(deferred, model, options) {
    var contentType = options.headers["Content-Type"];
    contentType && -1 !== contentType.indexOf("application/x-www-form-urlencoded") ? options.data = utils.param(model.toJSON()) : contentType && -1 !== contentType.indexOf("application/json") && (options.data = JSON.stringify(model.toJSON()));
    model.trigger("presave");
    return _wrapAPICall(deferred, model, options);
};

var _delete = function(deferred, model, options) {
    model.trigger("predelete");
    return _wrapAPICall(deferred, model, options);
};

var _getURL = function(model, options) {
    var url = _.result(options, "url") || _.result(model, "url") || _.result(model.config, "url");
    var dynamicURL = require("/carmalib/url")[url];
    return dynamicURL ? dynamicURL : url;
};

var _sync = function(method, model, options) {
    method = _METHOD_MAP[method];
    options = _.extend({}, options);
    options.method = method;
    if (model.config.urlMap && model.config.urlMap[options.method]) {
        var url = _.result(model.config.urlMap, options.method);
        options.url = _.template(url, model.toJSON());
    }
    options.url = _getURL(model, options);
    if (!options.url) {
        LOGGER.error("[REST Adapter] - Error: No url available");
        return Q.fcall(function() {
            throw new Error('A "url" property or function must be specified');
        });
    }
    var params = model.get("urlParams");
    options.url = _.template(options.url, params ? params : model.toJSON());
    params && model.unset("urlParams");
    LOGGER.info("[REST Adapter] - : " + options.url);
    options.headers = _.extend({}, _headers, options.headers);
    model.config.headers && (options.headers = _.extend({}, options.headers, model.config.headers));
    model.get("headers") && (options.headers = _.extend({}, options.headers, model.get("headers")));
    var deferred = Q.defer();
    if ("GET" === method) return _read(deferred, model, options);
    if ("POST" === method) return _save(deferred, model, options);
    if ("PUT" === method) return _save(deferred, model, options);
    if ("DELETE" === method) return _delete(deferred, model, options);
};

module.exports.sync = _sync;

module.exports.beforeModelCreate = function(config) {
    config = config || {};
    return config;
};

module.exports.afterModelCreate = function(Model) {
    Model = Model || {};
    Model.prototype.config.Model = Model;
    return Model;
};