0/0;

var Q = require("vendor/q"), utils = require("/utils/utils"), Uri = require("/vendor/uri"), SyncError = require("/alloy/utils/sync_error"), persistenceFactory = require("/alloy/persistence/persistence_factory");

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

var DEFAULT_TIME_OUT = 6e4, QOS_TIME_OUT = 1e4;

var _DEFAULT_HEADERS = {
    "Content-Type": "application/json;charset=UTF-8",
    Accept: "application/json",
    "User-Agent": Ti.userAgent
};

var _DEFAULT_OPTIONS = {
    localOnly: false,
    returnStoredData: true
};

var _METHOD_MAP = {
    read: "GET",
    create: "POST",
    update: "PUT",
    "delete": "DELETE"
};

var _createHttpClient = function(options) {
    return Ti.Network.createHTTPClient({
        cache: options.cache || false,
        enableKeepAlive: options.enableKeepAlive || true,
        timeout: options.timeout || DEFAULT_TIME_OUT
    });
};

var _getRemoteURL = function(model, options) {
    var url = options.url || _.result(model, "url") || model.config.url, isCollection = model.hasOwnProperty("models");
    model.config.urlMap && model.config.urlMap[options.method] && (url = model.config.urlMap[options.method]);
    var dynamicURL = require("/carmalib/url")[url];
    url = dynamicURL ? dynamicURL : url;
    var params = null;
    if (isCollection) params = _.extend({}, options.urlParams); else {
        params = _.extend({}, model.toJSON(), model.get("urlParams"), options.urlParams);
        model.unset("urlParams", {
            silent: true
        });
    }
    url = _.template(url, params);
    if ("GET" === options.method && options.data) {
        url = new Uri(url);
        for (var key in options.data) url.addQueryParam(key, options.data[key]);
        url = url.toString();
        delete options.data;
    }
    return url;
};

var parseSuccessResponse = function(client) {
    try {
        return JSON.parse(client.responseText);
    } catch (e) {
        LOGGER.info("[Sync Adapter] - reponse is not in JSON format");
    }
    return client.responseText;
};

var parseErrorResponse = function(client, response) {
    LOGGER.debug("[Sync Adapter] - http client on error");
    LOGGER.debug(response);
    var json = {};
    try {
        json = JSON.parse(client.responseText);
    } catch (e) {
        LOGGER.info("[Sync Adapter] - reponse is not in JSON format");
    }
    var error = SyncError.createErrorResponse(client.status, json ? json.errorCode || json.error || response.error : "", json ? json.description || json.error_description || response.error : "");
    LOGGER.error("[PERSISTENTREST] " + JSON.stringify(error));
    return error;
};

var _lastMonitorId = Date.now();

var _lastValue = true;

var Monitor = function() {
    return {
        id: Date.now(),
        start: function() {
            var self = this;
            self.timer = setTimeout(function() {
                self.stop(false);
            }, QOS_TIME_OUT);
        },
        stop: function(flag) {
            var self = this;
            if (self.timer) {
                clearTimeout(self.timer);
                self.timer = null;
                if (void 0 !== flag && null !== flag && self.id >= _lastMonitorId) {
                    _lastMonitorId = self.id;
                    if (_lastValue !== flag) {
                        _lastValue = flag;
                        Ti.App.fireEvent("carma:network.experience", {
                            value: flag
                        });
                    }
                }
            }
        }
    };
};

var _connect = function(model, options) {
    if (!Ti.Network.online) {
        LOGGER.error("[Sync Adapter] - offline, unable to make HTTP request");
        return SyncError.createErrorPromise(522, "offline", "no network connectivity");
    }
    var client = _createHttpClient(options), deferred = Q.defer();
    var monitorable = true !== model.config.background && "true" !== options.background;
    var monitor = monitorable ? Monitor() : null;
    monitor && monitor.start();
    client.onload = function() {
        monitor && monitor.stop(true);
        var successResponse = parseSuccessResponse(client);
        var merge = options.hasOwnProperty("merge") ? options.merge : model.config.merge;
        if (merge && options.cachedResults) {
            successResponse = require("/vendor/lodash").merge({}, options.cachedResults, successResponse);
            model.set(successResponse);
        }
        options.success && options.success(successResponse, client.status);
        var persistence = persistenceFactory[model.config.adapter.persistentType];
        if (persistence && !options.remoteOnly) {
            LOGGER.debug("[Sync Adapter] - persist server response");
            persistence.save(model, options);
        }
        deferred.resolve(model.toJSON());
    };
    client.onerror = function(response) {
        LOGGER.debug("[Sync Adapter] - API Error: " + client.responseText);
        if (monitor) {
            LOGGER.debug("[REST Adapter] - Experiencing bad internet connection... detected by the failure");
            var badConnection = response && response.code && ("1" === response.code.toString() || "2" === response.code.toString());
            monitor.stop(badConnection ? false : null);
        }
        var errorResponse = parseErrorResponse(client, response);
        if ("invalid_token" === errorResponse.errorCode) {
            Ti.App.fireEvent("carma:logout");
            return;
        }
        options.error && options.error(model, errorResponse, options);
        var handleError = model.config.handleError;
        if (handleError && -1 !== handleError.indexOf(errorResponse.errorCode)) {
            var persistence = persistenceFactory[model.config.adapter.persistentType];
            if (persistence) {
                LOGGER.debug("[Sync Adapter] - persist server response (error)");
                var idAttribute = model.config.idAttribute || "id", _id;
                !model.get("_id") && model.get(idAttribute) && (_id = model.get(idAttribute));
                model.set(errorResponse);
                persistence.save(model, options);
            }
            errorResponse.handleError = true;
        }
        deferred.reject(errorResponse);
    };
    var url = _getRemoteURL(model, options);
    LOGGER.debug("[Sync Adapter]: " + options.method + " " + url);
    client.open(options.method, url);
    LoggingService.log("network", url);
    for (var header in options.headers) client.setRequestHeader(header, options.headers[header]);
    client.send(options.data || null);
    LOGGER.debug("[Sync Adapter] - post data: " + options.data);
    return deferred.promise;
};

var _formatData = function(contentType, model) {
    if (!contentType) return model.toJSON();
    if (-1 !== contentType.indexOf("application/x-www-form-urlencoded")) {
        LOGGER.debug("[Sync Adapter] - post data as form format");
        return utils.param(model.toJSON());
    }
    if (-1 !== contentType.indexOf("application/json")) {
        LOGGER.debug("[Sync Adapter] - post data as json format");
        return JSON.stringify(model.toJSON());
    }
    return model.toJSON();
};

var _read = function(model, options) {
    LOGGER.debug("[Sync Adapter] - GET request");
    model.trigger("prefetch");
    var persistence = persistenceFactory[model.config.adapter.persistentType];
    var deferred = Q.defer();
    var cachedResults = null;
    persistence.fetch(model, options).then(function(offlineResults) {
        cachedResults = offlineResults;
        if (offlineResults || options.always_call_progress) {
            LOGGER.debug("[Sync Adapter] - progress with data from local storage");
            deferred.notify(offlineResults);
        }
    }).finally(function() {
        if (options.localOnly) {
            LOGGER.debug("[Sync Adapter] - fetch local data only");
            deferred.resolve(cachedResults);
        } else if (!Ti.Network.online && options.returnStoredData) {
            LOGGER.error("[Sync Adapter] - no network connection, get data from local storage");
            deferred.resolve(cachedResults);
        } else {
            options.cachedResults = cachedResults;
            _connect(model, options).then(function(apiResults) {
                LOGGER.debug("[Sync Adapter] - fullfill the promise with the response from API");
                deferred.resolve(apiResults);
            }).catch(function(error) {
                if (error && error.handleError) {
                    LOGGER.error("[Sync Adapter] - API Error, we handle it, do not fall back to local storage");
                    deferred.reject(error);
                    return;
                }
                LOGGER.error("[Sync Adapter] - API Error, fall back to the data from local storage");
                deferred.resolve(cachedResults);
            });
        }
    });
    return deferred.promise;
};

var _save = function(model, options) {
    LOGGER.debug("[Sync Adapter] - POST/PUT request");
    model.trigger("presave");
    var persistence = persistenceFactory[model.config.adapter.persistentType];
    if (options.localOnly) {
        LOGGER.debug("[Sync Adapter] - only store data locally");
        return persistence.save(model, options);
    }
    options.data = _formatData(options.headers["Content-Type"], model);
    return _connect(model, options);
};

var _delete = function(model, options) {
    LOGGER.debug("[Sync Adapter] - DELETE request");
    model.trigger("predelete");
    var persistence = persistenceFactory[model.config.adapter.persistentType];
    if (options.localOnly) {
        LOGGER.debug("[Sync Adapter] - only remove data locally");
        return persistence.remove(model, options);
    }
    return _connect(model, options);
};

var _personalisedCollection = function(model) {
    if (!model.config.adapter.personal) {
        model.config.adapter.collection = model.config.adapter.collectionName;
        return;
    }
    var resourceOwner = require("/services/resource_owner_service").singleton(), userId = resourceOwner.get("userId");
    userId && (model.config.adapter.collection = model.config.adapter.collectionName + "_" + userId);
};

var _attachOAuthToken = function(headers) {
    var resourceOwner = require("/services/resource_owner_service").singleton(), token = resourceOwner.get("token");
    token && (headers.Authorization = "Bearer " + token.token);
};

var _sync = function(method, model, options) {
    options = _.extend({}, _DEFAULT_OPTIONS, options);
    options.method = _METHOD_MAP[method];
    if (!options.method) {
        LOGGER.error("[Sync Adapter] - Error: Invalid HTTP method");
        return SyncError.createErrorPromise(401, "invalid_http_method", "Invalid HTTP methpd");
    }
    options.headers = _.extend({}, _DEFAULT_HEADERS, model.config.headers, options.headers);
    _personalisedCollection(model);
    _attachOAuthToken(options.headers);
    switch (options.method) {
      case "GET":
        return _read(model, options);

      case "POST":
        return _save(model, options);

      case "PUT":
        return _save(model, options);

      case "DELETE":
        return _delete(model, options);

      default:
        return;
    }
};

module.exports = {
    beforeModelCreate: function(config) {
        config = config || {};
        return config;
    },
    afterModelCreate: function(Model) {
        Model = Model || {};
        Model.prototype.config.Model = Model;
        return Model;
    },
    sync: _sync
};