"use strict";

var controllerOptions = [ "model", "collection", "events" ], delegateEventSplitter = /^(\S+)\s*(.*)$/, platformSpecificEventRegex = /(.*)\[(.+)\]/, platformSpecificEventSplitter = /\s*,\s*/, currentPlatform = "android", ui = require("/utils/ui");

var Controller = function(options) {
    this._sanityCheck();
    this._setOptions(options || {});
    this._postConstructArgs = arguments;
};

var isPlatformMatch = function(platform) {
    if (!platform) return false;
    platform = platform.toLowerCase();
    if (platform === currentPlatform) return true;
    if ("ios" === platform && ("iphone" === currentPlatform || "ipad" === currentPlatform)) return true;
    return false;
};

_.extend(Controller.prototype, {
    options: {
        CLOSE_WINDOW_TIME_OUT: 3e3
    },
    initialize: function() {},
    configure: function() {},
    closeWindow: function(immediate) {
        LOGGER && LOGGER.info("[Controller] - close window");
        if (immediate) {
            var window = ui.getWindow(this.getView());
            window && window.close && window.close();
            return;
        }
        var self = this;
        setTimeout(function() {
            var window = ui.getWindow(self.getView());
            window && window.close && window.close();
        }, this.options.CLOSE_WINDOW_TIME_OUT);
    },
    _destroyed: false,
    isDestroyed: function(argument) {
        return false | this._destroyed;
    },
    onBeforeHide: function() {},
    onDestroy: function() {},
    _sanityCheck: function() {
        if (this.render && !this.renderError) throw "Please provide function 'renderError' for handling rendering error";
        if (this._render && !this._renderError) throw "Please provide function '_renderError' for handling rendering error";
        return true;
    },
    _setOptions: function(options) {
        this.options && (options = _.extend({}, _.result(this, "options"), options));
        _.extend(this, _.pick(options, controllerOptions));
        this.options = options;
    },
    _attachEvent: function(viewId, eventName, method) {
        var view = null;
        if (-1 !== viewId.indexOf(".")) {
            var widgetView = viewId.split(".");
            view = this.getView(widgetView[0]).getView(widgetView[1]);
        } else view = this.getView(viewId);
        view && view.addEventListener(eventName, method);
    },
    _delegateEvents: function(events) {
        if (!events) return this;
        for (var key in events) {
            var method = events[key], match = key.match(delegateEventSplitter), eventName = match[1], viewId = match[2];
            _.isFunction(method) || (method = this[method]);
            if (!method || !eventName || !viewId) continue;
            method = _.bind(method, this);
            var platformSpecificEvents = eventName.match(platformSpecificEventRegex);
            if (platformSpecificEvents) {
                eventName = platformSpecificEvents[1];
                var platforms = platformSpecificEvents[2].split(platformSpecificEventSplitter);
                for (var i = 0; i < platforms.length; i++) {
                    var platform = platforms[i];
                    if (isPlatformMatch(platform)) {
                        LOGGER && LOGGER.debug("[Controller] - add platform[" + platform + "] specific event listener for: " + eventName);
                        this._attachEvent(viewId, eventName, method);
                    }
                }
            } else {
                LOGGER && LOGGER.debug("[Controller] - add cross platform event listener for: " + eventName + ", viewId: " + viewId);
                this._attachEvent(viewId, eventName, method);
            }
        }
    },
    postConstruct: function() {
        var view = this.getView();
        if (ui.isWindow(view)) if (view.addAppEventListener) LOGGER && LOGGER.debug("[EH] Already managing events for " + view.id); else {
            var eventManager = require("/ui/common/eventManager").createEventManager();
            eventManager.manageEvents(view.id, view);
            view.addEventListener("close", function handler() {
                LOGGER && LOGGER.debug("[CustomWindow] removing global event listeners: " + view.id);
                view.removeEventListener("close", handler);
                view.removeAllAppEventListeners();
            });
        } else {
            view.addAppEventListener = function(event, callback) {
                var _window = ui.getWindow(view);
                _window && _window.addAppEventListener(event, callback);
            };
            view.removeAppEventListener = function(event, callback) {
                var _window = ui.getWindow(view);
                _window && _window.removeAppEventListener(event, callback);
            };
        }
        this.configure.apply(this, this._postConstructArgs);
        this.initialize.apply(this, this._postConstructArgs);
        delete this._postConstructArgs;
        this._delegateEvents(this.events);
    }
});

Controller.extend = function($, protoProps, staticProps) {
    var parent = this;
    var child;
    child = protoProps && _.has(protoProps, "constructor") ? protoProps.constructor : function() {
        return parent.apply(this, arguments);
    };
    _.extend(child, parent, staticProps);
    var Surrogate = function() {
        this.constructor = child;
    };
    Surrogate.prototype = parent.prototype;
    child.prototype = new Surrogate();
    _.extend(child.prototype, $);
    var destroy = child.prototype.destroy;
    child.prototype.destroy = function() {
        LOGGER && LOGGER.debug("[Controller] - destroy method called");
        this.onDestroy.apply(this);
        destroy.apply(this);
        this._destroyed = true;
    };
    protoProps && _.extend(child.prototype, protoProps);
    child.__super__ = parent.prototype;
    return child;
};

module.exports = Controller;