"use strict";

var gps = null, platformConfig = require("/gps/gps_config"), locationValidator = require("/validator/validator_factory").currentLocation(), LAST_STORED_LOCATION = "_last_stored_location", _timer = null, watchId = null, lastServerUpdateTimestamp = 0, currentlySending = false, updateFreeRide = true, lastUserLocationRealTimestamp = null, lastUploadedLocation = null, managerStarted = false, geofenceHelper = require("/utils/GeofenceHelper");

var uploadLocation = function(location) {
    if (!location) return;
    var resourceOwner = require("services/resource_owner_service").singleton();
    if (!resourceOwner.hasLogin()) return;
    lastUploadedLocation || (lastUploadedLocation = Alloy.createModel("last_location"));
    lastUploadedLocation.clear();
    if (currentlySending) return;
    currentlySending = true;
    LOGGER.debug("[GPS Manager] - send new location to server...");
    var paramsObject = {
        urlParams: {
            userId: resourceOwner.get("userId")
        },
        latitude: location.lat,
        longitude: location.lon,
        accuracy: location.accuracy,
        timestamp: location.timestamp
    };
    location.address && (paramsObject.address = location.address);
    lastUploadedLocation.save(paramsObject).fin(function() {
        LOGGER.debug("[GPS Manager] - Finished sending location to server");
        currentlySending = false;
        geofenceHelper.gotLastKnownLocation();
    });
};

var getLastSavedLocation = function() {
    return Ti.App.Properties.getObject(LAST_STORED_LOCATION);
};

var updateLocationTimestamp = function(location) {
    if (location.userGenerated) LOGGER.debug("[GPS Manager] - Location is user-generated, not setting timestamp to NOW"); else {
        var collectionStart = locationValidator.getLocationCollectionBegin();
        if (location.timestamp && location.timestamp >= collectionStart) {
            LOGGER.debug("[GPS Manager] - Location is recent, setting timestamp to NOW");
            location.timestamp = Date.now();
        } else LOGGER.debug("[GPS Manager] - Location is old (" + location.timestamp + " < " + collectionStart + "), not setting timestamp to NOW");
    }
};

var processLocation = function(location) {
    LOGGER.debug("[GPS Manager] - Got new location");
    if (!locationValidator.isGPSPositionValid(location, location.userGenerated)) {
        LOGGER.debug("[GPS Manager] - New location is not valid");
        return;
    }
    if (lastUserLocationRealTimestamp && location.timestamp < lastUserLocationRealTimestamp) {
        LOGGER.debug("[GPS Manager] - location is older than latest known location");
        return;
    }
    lastUserLocationRealTimestamp = location.timestamp;
    LOGGER.debug("[GPS Manager] - location timestamp before update: " + location.timestamp);
    updateLocationTimestamp(location);
    LOGGER.debug("[GPS Manager] - location timestamp after update: " + location.timestamp);
    Ti.App.Properties.setObject(LAST_STORED_LOCATION, location);
    LOGGER.debug("[GPS Manager] - location is valid for usage");
    Ti.App.fireEvent("carma:valid.location.update", location);
};

var pollLocation = function() {
    try {
        LOGGER.debug("[GPS Manager] - In location update poller");
        process.backgroundDomain.enter();
        var location = getLastSavedLocation();
        if (!location) {
            LOGGER.debug("[GPS Manager] - No location available");
            return;
        }
        LOGGER.debug("[GPS Manager] - location timestamp before update: " + location.timestamp);
        updateLocationTimestamp(location);
        LOGGER.debug("[GPS Manager] - location timestamp after update: " + location.timestamp);
        if (location.timestamp >= lastServerUpdateTimestamp) {
            LOGGER.debug("[GPS Manager] - Uploading location to server");
            lastServerUpdateTimestamp = Date.now();
            uploadLocation(location);
        }
    } finally {
        process.backgroundDomain.exit();
    }
};

var getGpsService = function() {
    gps || (gps = require("/gps/gps_service"));
    return gps;
};

var stopGPS = function() {
    LOGGER.debug("[GPS Manager] - stop gps");
    LOGGER.debug("[GPS Manager] - Stopping location update poller");
    clearInterval(_timer);
    _timer = null;
    getGpsService().clearAllWatches();
    watchId = null;
};

var startGPS = function() {
    if (null !== watchId) return;
    LOGGER.debug("[GPS Manager] - start watch gps");
    watchId = getGpsService().watchPosition();
    pollLocation();
    var updateTimeMS = 1e3 * platformConfig.getServerUpdateTimeSecs();
    LOGGER.debug("[GPS Manager] - 1Starting location update poller every " + updateTimeMS + "ms");
    _timer = setInterval(pollLocation, updateTimeMS);
};

var stopLocationUpdate = function() {
    getGpsService().stopLocationUpdate();
};

module.exports = {
    init: function() {
        if (managerStarted) {
            startGPS();
            return;
        }
        managerStarted = true;
        var eventManagerModule = require("/ui/common/eventManager");
        eventManagerModule.setEventDomain("carma:location.update", process.backgroundDomain);
        eventManagerModule.setEventDomain("carma:location.disabled", process.backgroundDomain);
        eventManagerModule.setEventDomain("carma:debug.upload.location", process.backgroundDomain);
        eventManagerModule.setEventDomain("carma:valid.location.update", process.backgroundDomain);
        Ti.App.addAppEventListener("carma:location.disabled", function() {
            require("/ui/common/modal/gps_modal").turnOnGPSPrompt();
        });
        Ti.App.addAppEventListener("carma:debug.upload.location", uploadLocation);
        Ti.App.addAppEventListener("carma:location.update", processLocation);
        Ti.App.addAppEventListener("carma:life.cycle.pause", stopGPS);
        Ti.App.addAppEventListener("carma:life.cycle.resume", startGPS);
        Ti.App.addAppEventListener("carma:network.change", function(state) {
            if (state.connected) {
                var geoip = Alloy.createModel("geoip");
                geoip.fetch().then(function(data) {
                    GeoIPCountryCode = data.country_code;
                    GeoIPCountryCodeIsDefault = false;
                }).catch(function() {});
                updateFreeRide = true;
            }
        });
        Ti.App.addAppEventListener("carma:env.changed", function() {
            lastUploadedLocation = null;
        });
        updateFreeRide = true;
        TM.push(function() {
            LOGGER.debug("[GPS Manager] - start watch gps");
            watchId = getGpsService().watchPosition();
            var updateTimeMS = 1e3 * platformConfig.getServerUpdateTimeSecs();
            LOGGER.debug("[GPS Manager] - 2Starting location update poller every " + updateTimeMS + "ms");
            _timer = setInterval(pollLocation, updateTimeMS);
        }, process.backgroundDomain);
    },
    stop: stopGPS,
    lastLocation: getLastSavedLocation,
    stopLocationUpdate: stopLocationUpdate,
    locationUpdate: processLocation
};