"use strict";

function isGeolocationAccessible() {
    if (INSTRUMENT.location) {
        LOGGER.info("[GPS Manager] - Location instrumented, returning availability of " + INSTRUMENT.location.gps);
        return INSTRUMENT.location.gps;
    }
    return require("ma.car.ti.module.location").locationServicesEnabled;
    var authorization;
}

function normalizeLocation(lat, lon, accuracy, timestamp) {
    if (!validator.latlon().isLatLon(lat, lon)) throw "failed to inject location, invalid lat lon: " + lat + ", " + lon;
    return {
        lat: parseFloat(lat),
        lon: parseFloat(lon),
        accuracy: validator.number().isFloat(accuracy) ? parseInt(accuracy, 10) : -1,
        timestamp: validator.number().isInt(timestamp) ? parseInt(timestamp, 10) : Date.now()
    };
}

function locationDriftDebuging(plocation) {
    if (!plocation) return;
    var location = _.clone(plocation);
    if (previousLocation) {
        var p1 = {
            latitude: previousLocation.lat,
            longitude: previousLocation.lon
        }, p2 = {
            latitude: location.lat,
            longitude: location.lon
        };
        if (locationUtil.distance(p1, p2) / (location.timestamp - previousLocation.timestamp) > .05) {
            location.address = "GPS drift debug";
            Ti.App.fireEvent("carma:debug.upload.location", location);
        }
    }
    previousLocation = location;
}

function locationUpdate(e) {
    if (!(e && e.coords && e.coords.userGenerated || !INSTRUMENT.location)) {
        if (!INSTRUMENT.location.gps) {
            LOGGER.info("[GPS Manager] - GPS disabled, ignoring location");
            return;
        }
        if (INSTRUMENT.location.badgps) {
            LOGGER.info("[GPS Manager] - Simulating bad GPS, ignoring location");
            return;
        }
        e.coords = _.extend({}, INSTRUMENT.location);
        e.coords.timestamp = new Date().getTime();
        e.provider = {
            name: "instrumented"
        };
        e.success = true;
        delete e.coords.gps;
        delete e.coords.badgps;
        LOGGER.info("[GPS Manager] - GPS Mocked, setting location to " + JSON.stringify(e.coords));
    }
    if (!e.success) {
        LOGGER.error("location update error: " + (e.error || "unknown"));
        return;
    }
    if (!e.coords) return;
    true && e.provider && LOGGER.debug("location update from provider: " + (e.provider.name || "unknown"));
    try {
        var location = normalizeLocation(e.coords.latitude, e.coords.longitude, e.coords.accuracy, e.coords.timestamp);
        location.accuracy = e.coords.accuracy;
        e.coords.userGenerated && (location.userGenerated = true);
        require("gps/gps_manager").locationUpdate(location);
        locationDriftDebuging(location);
    } catch (e) {
        LOGGER.error(e);
    }
}

function headingUpdated() {}

function locationUpdatePaused() {
    LOGGER.debug("[GPS Manager] - location update paused");
}

function locationUpdateResumed() {
    LOGGER.debug("[GPS Manager] - location update resumed");
}

function addEventListeners() {
    if (watchIds.length > 0) {
        LOGGER.debug("[GPS Manager] - already listening to gps events...");
        return;
    }
    LOGGER.debug("[GPS Manager] - add gps event listeners");
    platformConfig.config();
    require("ma.car.ti.module.location").addEventListener("location", locationUpdate);
}

function removeEventListeners() {
    if (watchIds.length > 0) {
        LOGGER.debug("[GPS Manager] - unable to remove gps event listeners, maybe some other views are still listening...");
        return;
    }
    LOGGER.debug("[GPS Manager] - remove gps event listeners");
    var module = require("ma.car.ti.module.location");
    module.removeEventListener("location", locationUpdate);
    module.stopLocationUpdate();
}

function stopLocationUpdate() {
    require("ma.car.ti.module.location").stopLocationUpdate();
}

function addWatch() {
    locationValidator.setLocationCollectionBegin();
    addEventListeners();
    var watchId = Date.now();
    watchIds.push(watchId);
    LOGGER.debug("[GPS Manager] - gps watch id: " + watchId);
    return watchId;
}

function removeWatch(watchId) {
    var index = watchIds.indexOf(watchId);
    if (-1 === index) {
        LOGGER.error("watch id not found: " + watchId);
        return;
    }
    watchIds.splice(index, 1);
    removeEventListeners();
    locationValidator.setLocationCollectionEnd();
    LOGGER.debug("[GPS Manager] - cleared gps watch id: " + watchId);
}

var platformConfig = require("/gps/gps_config"), validator = require("validator/validator_factory"), locationValidator = require("/validator/validator_factory").currentLocation(), watchIds = [], previousLocation = null, locationUtil = require("/gps/location");

module.exports = {
    isWatching: function() {
        return watchIds.length > 0;
    },
    watchPosition: function() {
        if (!isGeolocationAccessible()) {
            LOGGER.warn("location disabled");
            Ti.App.fireEvent("carma:location.disabled");
        }
        return addWatch();
    },
    clearWatch: function(watchId) {
        removeWatch(watchId);
    },
    clearAllWatches: function() {
        watchIds.forEach(function(watchId) {
            removeWatch(watchId);
        });
    },
    injectLocation: function(lat, lon, accuracy, timestamp) {
        locationUpdate({
            success: true,
            coords: {
                userGenerated: true,
                latitude: lat,
                longitude: lon,
                accuracy: accuracy,
                timestamp: timestamp
            },
            provider: {
                name: "Manual Injection"
            }
        });
    },
    stopLocationUpdate: stopLocationUpdate,
    isLocationServiceEnabled: isGeolocationAccessible
};