"use strict";

var Q = require("vendor/q");

var assert = require("/utils/assert");

var purl = require("/utils/purl");

var util = require("ma.car.ti.module.util");

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

var GROCODE_API = "https://maps.googleapis.com/maps/api/geocode/json?";

var PLACES_AUTOCOMPLETE_API = "https://maps.googleapis.com/maps/api/place/autocomplete/json?";

var PLACES_DETAILS_API = "https://maps.googleapis.com/maps/api/place/details/json?";

var DIRECTIONS_API = "http://maps.googleapis.com/maps/api/directions/json?";

var STATICMAPS_API = "https://maps.googleapis.com/maps/api/staticmap?";

var API_KEY = "AIzaSyDaPs91xupRUM8VSeVpDH5pGpUv01JgzNI";

var CLIENT_ID = "gme-avego";

var DEFAULT_RADIUS = 5e5;

var _getAddressComponent = function(addressComponents, type, isLong) {
    for (var i = 0; i < addressComponents.length; i++) {
        var component = addressComponents[i];
        if (-1 !== _.indexOf(component.types, type)) return isLong ? component.long_name : component.short_name;
    }
    return null;
};

var _getGeocodeResults = function(results) {
    if (!results || 0 === results.length) return [];
    var addresses = _.map(results, function(result) {
        return {
            address: result.description,
            reference: result.reference
        };
    });
    return _.uniq(addresses, false, function(address) {
        return address.reference;
    });
};

var _placeSearch = function(options) {
    var deferred = Q.defer();
    assert.required(options, [ "input" ]);
    var params = {
        input: options.input,
        sensor: false,
        key: API_KEY
    };
    if (options.latitude && options.longitude) {
        params.location = options.latitude + "," + options.longitude;
        params.radius = options.radius || DEFAULT_RADIUS;
    }
    options.country && (params.components = "country:" + options.country);
    var url = purl.purl(PLACES_AUTOCOMPLETE_API).toURL(params);
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            var response = JSON.parse(this.responseText);
            var status = response.status;
            if ("OK" === status) {
                var items = response.predictions ? response.predictions : [];
                items = options.items ? options.items.concat(items) : items;
                if (items.length > 5) {
                    deferred.resolve({
                        type: "placeSearch",
                        result: _getGeocodeResults(items),
                        options: options
                    });
                    return;
                }
                if (options.country) {
                    delete options.country;
                    delete options.location;
                    options.items = items;
                    deferred.resolve(_placeSearch(options));
                    return;
                }
                if (items.length > 0) {
                    deferred.resolve({
                        type: "placeSearch",
                        result: _getGeocodeResults(items),
                        options: options
                    });
                    return;
                }
                deferred.reject({
                    type: "placeSearch",
                    status: "ZERO_RESULTS",
                    options: options
                });
            } else deferred.reject({
                type: "placeSearch",
                status: status,
                options: options
            });
        },
        onerror: function() {
            deferred.reject({
                type: "placeSearch",
                status: "NETWORK_ERROR",
                options: options
            });
        },
        timeout: 2e4
    });
    xhr.setRequestHeader("Content-Type", "application/json; charset=utf-8");
    xhr.open("GET", url);
    LoggingService.log("network", url);
    xhr.send(null);
    return deferred.promise;
};

var _placeDetails = function(options) {
    var deferred = Q.defer();
    assert.required(options, [ "reference" ]);
    var url = purl.purl(PLACES_DETAILS_API).toURL({
        reference: options.reference,
        sensor: false,
        key: API_KEY
    });
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            var response = JSON.parse(this.responseText);
            var status = response.status;
            if ("OK" === status) {
                var item = response.result;
                var result = {
                    lat: item.geometry.location.lat.toFixed(5),
                    lon: item.geometry.location.lng.toFixed(5),
                    address: item.formatted_address,
                    country: _getAddressComponent(item.address_components, "country")
                };
                deferred.resolve({
                    type: "placeDetails",
                    result: result,
                    options: options
                });
            } else deferred.reject({
                type: "placeDetails",
                status: status,
                options: options
            });
        },
        onerror: function() {
            deferred.reject({
                type: "placeDetails",
                status: "NETWORK_ERROR",
                options: options
            });
        },
        timeout: 2e4
    });
    xhr.setRequestHeader("Content-Type", "application/json; charset=utf-8");
    xhr.open("GET", url);
    LoggingService.log("network", url);
    xhr.send(null);
    return deferred.promise;
};

var _geocode = function(options) {
    var deferred = Q.defer();
    assert.required(options, [ "address" ]);
    var url = purl.purl(GROCODE_API).toURL({
        address: options.regionCode ? options.address + "" + options.regionCode : options.address,
        client: CLIENT_ID,
        sensor: false
    });
    var signedURL = util.urlSigner(url);
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            var response = JSON.parse(this.responseText);
            var status = response.status;
            if ("OK" === status) {
                var result = _.map(response.results, function(item) {
                    return {
                        address: item.formatted_address,
                        lat: item.geometry.location.lat.toFixed(5),
                        lon: item.geometry.location.lng.toFixed(5)
                    };
                });
                deferred.resolve({
                    type: "geocode",
                    result: result,
                    options: options
                });
            } else deferred.reject({
                type: "geocode",
                status: status,
                options: options
            });
        },
        onerror: function() {
            deferred.reject({
                type: "geocode",
                status: "NETWORK_ERROR",
                options: options
            });
        },
        timeout: 2e4
    });
    xhr.open("GET", signedURL);
    LoggingService.log("network", signedURL);
    xhr.send(null);
    return deferred.promise;
};

var _reverseGeocode = function(options) {
    var deferred = Q.defer();
    assert.required(options, [ "latitude", "longitude" ]);
    var key = options.latitude + "," + options.longitude;
    var caches = Ti.App.Properties.getObject("reverseGeocodes", {});
    var cache = caches[key];
    deferred.notify(cache);
    var url = purl.purl(GROCODE_API).toURL({
        latlng: key,
        client: CLIENT_ID,
        sensor: false
    });
    var signedURL = util.urlSigner(url);
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            var response = JSON.parse(this.responseText);
            var status = response.status;
            if ("OK" === status) {
                var item = response.results[0];
                var result = {
                    address: item.formatted_address,
                    country: _getAddressComponent(item.address_components, "country"),
                    countryLong: _getAddressComponent(item.address_components, "country", true),
                    locality: _getAddressComponent(item.address_components, "locality", true)
                };
                caches[key] = result;
                Ti.App.Properties.setObject("reverseGeocodes", caches);
                deferred.resolve({
                    type: "reverseGeocode",
                    result: result,
                    options: options
                });
            } else deferred.reject({
                type: "reverseGeocode",
                status: status,
                options: options
            });
        },
        onerror: function() {
            deferred.reject({
                type: "reverseGeocode",
                status: "NETWORK_ERROR",
                options: options
            });
        },
        timeout: 2e4
    });
    xhr.open("GET", signedURL);
    LoggingService.log("network", signedURL);
    xhr.send(null);
    return deferred.promise;
};

var _directions = function(options) {
    var deferred = Q.defer();
    assert.required(options, [ "origin", "destination" ]);
    var url = purl.purl(DIRECTIONS_API).toURL({
        origin: options.origin.lat + "," + options.origin.lon,
        destination: options.destination.lat + "," + options.destination.lon,
        mode: "driving",
        sensor: false,
        client: CLIENT_ID
    });
    var signedURL = util.urlSigner(url);
    var xhr = Ti.Network.createHTTPClient({
        onload: function() {
            var response = JSON.parse(this.responseText);
            var status = response.status;
            if ("OK" === status) {
                var item = response.results[0];
                var result = {
                    points: item.routes.overview_polyline.points
                };
                deferred.resolve({
                    type: "directions",
                    result: result,
                    options: options
                });
            } else deferred.reject({
                type: "directions",
                status: status,
                options: options
            });
        },
        onerror: function() {
            deferred.reject({
                type: "directions",
                status: "NETWORK_ERROR",
                options: options
            });
        },
        timeout: 2e4
    });
    xhr.open("GET", signedURL);
    LoggingService.log("network", signedURL);
    xhr.send(null);
    return deferred.promise;
};

var _staticMapURL = function(options) {
    assert.required(options, [ "origin", "destination" ]);
    var centerLat = parseFloat((parseFloat(options.origin.lat) + parseFloat(options.destination.lat)) / 2).toFixed(5);
    var centerLon = parseFloat((parseFloat(options.origin.lon) + parseFloat(options.destination.lon)) / 2).toFixed(5);
    var url = purl.purl(STATICMAPS_API).toURL({
        center: centerLat + "," + centerLon,
        size: options.size.width + "x" + options.size.height,
        sensor: false,
        client: CLIENT_ID
    });
    url += "&markers=color:blue%7Clabel:O%7C" + options.origin.lat + "," + options.origin.lon;
    url += "&markers=color:blue%7Clabel:D%7C" + options.destination.lat + "," + options.destination.lon;
    return util.urlSigner(url);
};

module.exports = {
    geocode: function(options) {
        return _geocode(options);
    },
    reverseGeocode: function(options) {
        return _reverseGeocode(options);
    },
    placeSearch: function(options) {
        return _placeSearch(options);
    },
    placeDetails: function(options) {
        return _placeDetails(options);
    },
    directions: function(options) {
        return _directions(options);
    },
    staticMapURL: function(options) {
        return _staticMapURL(options);
    },
    distance: function(p1, p2, miles) {
        var rad = function(x) {
            return x * Math.PI / 180;
        };
        var R = 6371;
        var dLat = rad(p2.latitude - p1.latitude);
        var dLong = rad(p2.longitude - p1.longitude);
        var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) + Math.cos(rad(p1.latitude)) * Math.cos(rad(p2.latitude)) * Math.sin(dLong / 2) * Math.sin(dLong / 2);
        var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        var d = R * c;
        var km = d.toFixed(3);
        if (miles) return Math.round(1e3 * km * 621371e-9 * 10) / 10;
        return km;
    }
};