"use strict";

var i18n = require("i18n/i18n"), BoardingDisplayHelper = require("./BoardingDisplayHelper");

var UNREGISTERED_USER_ID = -2;

var toOrdinalNumberString = function(num) {
    if (1 === num) return "First";
    if (new RegExp(/^[0-9]*1$/g).test(num) && num % 100 != 11) return num + "st";
    if (new RegExp(/^[0-9]*2$/g).test(num) && num % 100 != 12) return num + "nd";
    if (new RegExp(/^[0-9]*3$/g).test(num) && num % 100 != 13) return num + "rd";
    return num + "th";
};

var SummaryBadgeWidgetFactory = function(profileService) {
    if (!profileService) throw new Error("ProfileService is required!");
    var displayHelper = new BoardingDisplayHelper(profileService);
    return {
        createErrorBadge: function(summary, badgeMetrics) {
            var errorTranslator = require("/utils/errorTranslator"), errorImage = "/images/TripSummary_FailedTrip.png", errorCode = summary.drive && summary.drive.state ? summary.drive.state.errorCode ? summary.drive.state.errorCode : summary.drive.state.errorMessage : null;
            if (summary.unrealisticCause && !Ti.App.Properties.getBool("disableUnlikely", false)) {
                errorCode = "unlikely_carpool";
                errorImage = "/images/TripSummary_UnlikelyRideshare.png";
            }
            var error = errorTranslator.translate("drive_error", errorCode);
            var widget = Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: errorImage,
                badgeMetrics: badgeMetrics
            });
            return summary.unrealisticCause && !Ti.App.Properties.getBool("disableUnlikely", false) ? {
                controller: widget,
                name: error.title,
                description: error.message
            } : {
                controller: widget,
                name: error.message,
                description: ""
            };
        },
        createOfflineBadge: function(summary, badgeMetrics) {
            var errorTranslator = require("/utils/errorTranslator"), error = errorTranslator.translate("drive_error", "pending_data");
            var widget = Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: "/images/offline.png",
                badgeMetrics: badgeMetrics
            });
            return {
                controller: widget,
                name: error.title,
                description: error.message
            };
        },
        createAllOKBadge: function(summary, badgeMetrics) {
            var count = summary.carpoolCount;
            count || (count = 0);
            var widget = Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: "/images/TripSummary_CompletedTrip.png",
                badgeMetrics: badgeMetrics
            }), preferences = require("services/cached_preferences_service").singleton(), unit = L("MILES" === preferences.getPreferredDistanceUnit() ? "unit_mile" : "widget_rating_km_label"), distance = i18n.formatDistanceFromMeters(summary.distance);
            return {
                shareContent: String.format(L("widget_summary_share_carpool"), distance + " " + unit),
                shareSubject: String.format(L("widget_summary_share_carpool_subject"), distance + " " + unit),
                controller: widget,
                name: L(0 === summary.rides.length ? "widget_summary_solo_trip_complete_title" : "widget_summary_trip_complete_title"),
                description: count > 1 ? String.format(L("widget_summary_trip_complete_des"), count) : String.format(L("widget_summary_first_trip_complete_des"), count)
            };
        },
        createTollRewardBadge: function(summary, reward, badgeMetrics) {
            var widget = reward.iconUrl ? Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: reward.iconUrl,
                badgeMetrics: badgeMetrics
            }) : Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: "/images/TripSummary_TollReward.png",
                badgeMetrics: badgeMetrics
            });
            var tollReward = _.filter(summary.rewardTotals, function(reward) {
                return "TOLL" === reward.type;
            }).shift() || {};
            return {
                shareContent: String.format(L("widget_summary_share_toll"), reward.amount),
                shareSubject: L("widget_summary_share_toll_subject"),
                controller: widget,
                name: String.format(L("widget_summary_toll_reward_title"), reward.amount),
                description: tollReward.amount ? String.format(L("widget_summary_toll_reward_des"), tollReward.amount) : ""
            };
        },
        createParkingRewardBadge: function(summary, reward, badgeMetrics) {
            var widget = reward.iconUrl ? Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: reward.iconUrl,
                badgeMetrics: badgeMetrics
            }) : Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: "/images/TripSummary_ParkingReward_Generic.png",
                badgeMetrics: badgeMetrics
            });
            var parkingReward = _.filter(summary.rewardTotals, function(reward) {
                return "PARKING" === reward.type;
            }).shift() || {};
            return {
                shareContent: String.format(L("widget_summary_share_parking"), reward.amount),
                shareSubject: L("widget_summary_share_parking_subject"),
                controller: widget,
                name: String.format(L("widget_summary_parking_reward_title"), reward.amount),
                description: parkingReward.amount ? String.format(L("widget_summary_parking_reward_des"), parkingReward.amount) : ""
            };
        },
        createInviteeRewardBadge: function(reward, badgeMetrics) {
            var widget = reward.iconUrl ? Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: reward.iconUrl,
                badgeMetrics: badgeMetrics
            }) : Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: "/images/TripSummary_TripMilestone.png",
                badgeMetrics: badgeMetrics
            });
            var userservice = require("/services/user_service"), inviteCode = userservice.getCachReferralCode(), relatedUsername = reward.relatedUsername || L("widget_summary_unknown_user");
            return {
                shareContent: L("widget_summary_share_invitee_signup") + inviteCode,
                shareSubject: L("widget_summary_share_invitee_signup_subject"),
                controller: widget,
                name: String.format(L("widget_summary_invitee_reward_title"), reward.amount, relatedUsername),
                description: L("widget_summary_invitee_reward_des")
            };
        },
        createAchievementBadge: function(achievement, badgeMetrics) {
            var widget = achievement.iconUrl ? Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: achievement.iconUrl,
                badgeMetrics: badgeMetrics
            }) : Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: "/images/TripSummary_TripMilestone.png",
                badgeMetrics: badgeMetrics
            });
            var resourceOwner = require("services/resource_owner_service").singleton(), count = require("services/user_service").cachedCount("rides", resourceOwner.get("userId"));
            count || (count = 0);
            return {
                shareContent: String.format(L("widget_summary_share_achievement"), toOrdinalNumberString(count)),
                shareSubject: String.format(L("widget_summary_share_achievement_subject"), L(toOrdinalNumberString(count) + count > 0 ? "native_share_carma_carpools" : "native_share_carma_carpool")),
                controller: widget,
                name: achievement.title,
                description: achievement.description
            };
        },
        createBuddyBadge: function(summary, userId, driverPhoneNumber, badgeMetrics) {
            var widget = Alloy.createWidget("ma.car.app.boardingv6", "summaryBadge", {
                localImage: "/images/default_photo_male-rounded.png",
                badgeMetrics: badgeMetrics
            });
            if (userId == UNREGISTERED_USER_ID && driverPhoneNumber) return {
                shareContent: "",
                shareSubject: "",
                controller: widget,
                name: driverPhoneNumber,
                description: String.format(L("widget_summary_rider_des"), driverPhoneNumber)
            };
            if (!userId) return {
                shareContent: "",
                shareSubject: "",
                controller: widget,
                name: L("widget_summary_unknown_user"),
                description: ""
            };
            var profile = profileService.getCachedProfile(userId);
            var cachedImage = displayHelper.getCachedUserAvatarImage(userId);
            if (cachedImage) {
                widget.setLocalImage(cachedImage);
                cachedImage = null;
            } else displayHelper.getUserAvatarImage(userId).then(function(image) {
                widget.setLocalImage(image);
            }).catch(function(e) {
                console.log(e);
            });
            var preferences = require("services/cached_preferences_service").singleton(), unit = L("MILES" === preferences.getPreferredDistanceUnit() ? "unit_mile" : "widget_rating_km_label"), distance = i18n.formatDistanceFromMeters(summary.distance), alias = profile ? profile.alias : L("widget_summary_unknown_user");
            if (profile) {
                var pos = alias.lastIndexOf(" ");
                pos > 0 && (alias = alias.substring(0, pos));
            }
            return {
                shareContent: String.format(L("widget_summary_share_user"), distance + " " + unit, alias),
                shareSubject: String.format(L("widget_summary_share_user_subject"), alias),
                controller: widget,
                name: profile ? profile.alias : L("widget_summary_unknown_user"),
                description: profile ? String.format(L("widget_summary_rider_des"), alias) : ""
            };
        }
    };
};

module.exports = SummaryBadgeWidgetFactory;