"use strict";

var q = require("vendor/q");

var moment = require("vendor/moment");

var i18n = require("i18n/i18n");

var FACEBOOK_LOGIN_AVAILABLE_DOMAIN = "car.ma";

var FACEBOOK_TRIP_SHARE_URL_TEMPLATE = "/s/det/viewtripd?uid=<%= uid %>&tripId=<%= tripId %>&ts=<%= timestamp %>";

var _buildScheduleTitle = function(user, trip, forFeed) {
    var key = "trip.share";
    key += forFeed ? ".feed" : "";
    key += "." + trip.type.toLowerCase();
    key += forFeed ? "" : "m" === user.gender ? ".he" : ".she";
    var weekdays = true, time = null, date = null;
    if ("ONE_OFF" === trip.schedule.type) {
        key += ".on";
        time = i18n.formatTime(moment(trip.schedule.startTime));
        date = i18n.formatLongDate(moment(trip.schedule.startTime));
    } else {
        var daysOfOperation = trip.schedule.daysOfOperationv2;
        for (var i = 0; i < daysOfOperation.length; i++) {
            weekdays = weekdays && ((0 === i || 6 === i) && !daysOfOperation[i] || 0 !== i && 6 !== i && daysOfOperation[i]);
            if (!weekdays) {
                key += ".weekdays";
                break;
            }
        }
        key += weekdays ? ".weekdays" : ".title";
    }
    return String.format(L(key), trip.origin.address, trip.destination.address, time, date);
};

var _buildSchedule = function(user, trip) {
    var schedule = {};
    schedule.type = "getavego:schedule";
    schedule.title = _buildScheduleTitle(user, trip);
    schedule.feedTitle = _buildScheduleTitle(user, trip, true);
    schedule.url = "http://" + FACEBOOK_LOGIN_AVAILABLE_DOMAIN + _.template(FACEBOOK_TRIP_SHARE_URL_TEMPLATE, {
        uid: user.userId,
        tripId: trip.id,
        timestamp: trip.schedule.effectiveTime
    });
    schedule.picture = require("bootstrap/env").getCDNUrl().replace("https", "http") + "images/carma.png";
    schedule.image = schedule.picture;
    schedule.data = {};
    schedule.data.origin = {
        latitude: trip.origin.latitude,
        longitude: trip.origin.longitude
    };
    schedule.data.destination = {
        latitude: trip.destination.latitude,
        longitude: trip.destination.longitude
    };
    schedule.data.time = i18n.formatTime(moment(trip.schedule.startTime));
    schedule.data.distance = i18n.formatDistanceFromMeters(trip.tripDistance);
    schedule.data.price = "$" + trip.estimatedCost;
    return schedule;
};

var _getMapImageURLRequestParams = function(schedule) {
    var params = {
        data: {
            size: {
                height: 300,
                width: 300
            }
        }
    };
    schedule.data.origin && (params.data.origin = {
        lat: parseFloat(schedule.data.origin.latitude).toFixed(5),
        lon: parseFloat(schedule.data.origin.longitude).toFixed(5)
    });
    schedule.data.destination && (params.data.destination = {
        lat: parseFloat(schedule.data.destination.latitude).toFixed(5),
        lon: parseFloat(schedule.data.destination.longitude).toFixed(5)
    });
    schedule.data.origin && schedule.data.destination && (params.data.path = {
        fromLat: parseFloat(schedule.data.origin.latitude).toFixed(5),
        fromLon: parseFloat(schedule.data.origin.longitude).toFixed(5),
        toLat: parseFloat(schedule.data.destination.latitude).toFixed(5),
        toLon: parseFloat(schedule.data.destination.longitude).toFixed(5)
    });
    return params;
};

var _shareable = function(trip) {
    var schedule = null;
    var promise = q.fcall(function() {
        var user = require("services/resource_owner_service").singleton().toJSON();
        schedule = _buildSchedule(user, trip);
    });
    promise = promise.then(function() {
        return Alloy.createModel("share_request", {
            tripId: trip.id,
            method: "FACEBOOK"
        }).save();
    });
    promise = promise.then(function(data) {
        data.acceptUrl && (schedule.url = data.acceptUrl);
    });
    promise = promise.then(function() {
        return Alloy.createModel("points_route").route({
            originLat: schedule.data.origin.latitude,
            originLon: schedule.data.origin.longitude,
            destinationLat: schedule.data.destination.latitude,
            destinationLon: schedule.data.destination.longitude
        }).then(function(data) {
            if (data) {
                schedule.data.route = data.route;
                schedule.data.path = data.path;
                return;
            }
            throw {
                code: "no_route_found"
            };
        });
    });
    promise = promise.then(function() {
        return schedule;
    });
    return promise;
};

module.exports = {
    update: function(trip) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        var userId = resourceOwner.get("userId");
        return Alloy.createModel("trip", trip).update(userId);
    },
    share: function(trip) {
        return _shareable(trip).then(function(schedule) {
            return require("facebook/facebook_manager").publish("getavego:set_up", "getavego:schedule", schedule);
        });
    },
    bestMatches: function(pageSize) {
        return Alloy.createModel("trip_matches", {
            sortBy: "BEST_MATCH",
            searchTargetType: "RIDE_OR_DRIVE",
            includeTypeCounts: false,
            pageSize: void 0 !== pageSize && null !== pageSize ? pageSize : 4,
            pageNum: 1,
            nextTripTimestamp: Date.now() - 9e5
        }).bestMatches().then(function(data) {
            var nextTrip = data && data.tripMatchedAgainst && data.tripMatchedAgainst.baseTrip || {};
            var resourceOwner = require("services/resource_owner_service").singleton();
            resourceOwner.set("nextTrip", nextTrip);
            resourceOwner.save();
            return data;
        });
    },
    matches: function(searchTargetType, pageNum) {
        return Alloy.createModel("trip_matches", {
            sortBy: "BEST_MATCH",
            pageSize: 4,
            searchTargetType: searchTargetType,
            pageNum: pageNum ? pageNum : 1,
            nextTripTimestamp: Date.now() - 9e5
        }).fetch();
    },
    groupingKey: function(nextTrip, matchedTrip) {
        if (!nextTrip || !nextTrip.schedule) return null;
        if (!matchedTrip || !matchedTrip.schedule) return null;
        if (Math.abs(matchedTrip.schedule.effectiveTime - nextTrip.schedule.effectiveTime) > 72e5) return null;
        return nextTrip.id + "-" + nextTrip.schedule.effectiveTime;
    },
    getNextnTrips: function(numTrips) {
        return Alloy.createModel("trips_nextn").getNextnTrips({
            numTrips: numTrips
        });
    }
};