"use strict";

var Q = require("/vendor/q"), windowManager = require("/ui/windowManager");

var visitViews = function(w, visitor) {
    if (!w) return;
    visitor.startVisitWindow && visitor.startVisitWindow(w);
    w.children && w.children.forEach(function(child) {
        visitViews(child, visitor);
    });
    w.views && w.views.forEach(function(child) {
        visitViews(child, visitor);
    });
    w.windows && w.windows.forEach(function(child) {
        visitViews(child, visitor);
    });
    w.headerView && visitViews(w.headerView, visitor);
    w.sections && w.sections.forEach(function(child) {
        visitViews(child, visitor);
    });
    w.footerView && visitViews(w.footerView, visitor);
    w.rows && w.rows.forEach(function(child) {
        visitViews(child, visitor);
    });
    w.tabs && w.tabs.forEach(function(child) {
        visitViews(child, visitor);
    });
    w.window && visitViews(w.window, visitor);
    visitor.endVisitWindow && visitor.endVisitWindow(w);
};

var initialize = function() {
    var Mocha = require("/vendor/ti-mocha");
    mocha.setup({
        grep: INSTRUMENT.test.grep,
        reporter: "ti-spec-splinter",
        ui: "bdd"
    });
    var config = require("specs-ui/config");
    var runner = mocha.run(function() {
        var stats = _.extend({}, {
            tests: 0,
            passes: 0,
            failures: 0
        }, runner ? runner.stats : {});
        var reportFile = null;
        if (INSTRUMENT.test.report) {
            reportFile = Ti.Filesystem.getFile(Ti.Filesystem.tempDirectory, INSTRUMENT.test.report);
            var result = JSON.stringify(stats, 0, 4);
            console.log("####################################################################################################################");
            console.log("####################################################################################################################");
            console.log("Test Report: " + INSTRUMENT.test.report);
            console.log(result);
            console.log("####################################################################################################################");
            console.log("####################################################################################################################");
            reportFile.write(result);
        }
    });
};

var _getTopWindow = function() {
    var currentWindow = windowManager.getCurrentWindow();
    return currentWindow ? currentWindow : TAB_GROUP;
};

var _clientToScreen = function(point, wnd) {
    var parent = wnd.getParent();
    while (parent) {
        point.x += parent.rect.x;
        point.y += parent.rect.y;
        parent = parent.getParent();
    }
    return point;
};

var _isVisible = function(wnd) {
    if (wnd.window) return wnd.active;
    if (false === wnd.visible || 0 === wnd.opacity) return false;
    var topWindow = _getTopWindow();
    if (!topWindow) return false;
    if (!wnd || !wnd.rect) return true;
    var screenCoord = _clientToScreen({
        x: wnd.rect.x,
        y: wnd.rect.y
    }, wnd);
    if (!screenCoord || screenCoord.x >= Ti.Platform.displayCaps.platformWidth || screenCoord.y >= Ti.Platform.displayCaps.platformHeight || screenCoord.x + wnd.rect.width < 0 || screenCoord.y + wnd.rect.height < 0) return false;
    return true;
};

var _dumpWindows = function(currentWindow) {
    if (!currentWindow) return;
    var indent = "";
    visitViews(currentWindow, {
        startVisitWindow: function(wnd) {
            var attr = "";
            wnd.title && (attr += " title: '" + wnd.title + "'");
            wnd.text && (attr += " text: '" + wnd.text + "'");
            wnd.image && (attr += " image: '" + wnd.image + "'");
            wnd.rect && (attr += " rect: '" + JSON.stringify({
                x: wnd.rect.x,
                y: wnd.rect.y,
                width: wnd.rect.width,
                height: wnd.rect.height
            }) + "'");
            attr += " visible: '" + wnd.visible + "'";
            attr += " opacity: '" + wnd.opacity + "'";
            LOGGER.error(indent + wnd + "[" + attr + "] {");
            indent += "    ";
        },
        endVisitWindow: function(wnd) {
            indent = indent.substring(0, indent.length - 4);
            LOGGER.error(indent + "}");
        }
    });
};

module.exports = {
    getTopWindow: _getTopWindow,
    dumpWindows: function(wnd) {
        _dumpWindows(wnd ? wnd : this.getTopWindow());
    },
    isVisible: _isVisible,
    clientToScreen: function(point, wnd) {
        return _clientToScreen(point, wnd);
    },
    findView: function(match, allowInvisible, dontThrowIfNotFound, currentWindow) {
        var found = null;
        var stack = [];
        var isStackVisible = true;
        var self = this;
        currentWindow || (currentWindow = this.getTopWindow());
        currentWindow && visitViews(currentWindow, {
            startVisitWindow: function(wnd) {
                stack.push(isStackVisible);
                isStackVisible &= self.isVisible(wnd);
                !found && (isStackVisible || allowInvisible) && match(wnd) && (found = wnd);
            },
            endVisitWindow: function(wnd) {
                isStackVisible = stack.pop();
            }
        });
        if (!found && !dontThrowIfNotFound) {
            this.dumpWindows(currentWindow);
            throw new Error("Could not find window [allowInvisible: " + !!allowInvisible + "]");
        }
        return found;
    },
    findViewById: function(id, allowInvisible, dontThrowIfNotFound, currentWindow) {
        LOGGER.info("############################################## findViewById: " + id);
        return this.findView(function(wnd) {
            return wnd.id === id;
        }, allowInvisible, dontThrowIfNotFound, currentWindow);
    },
    findViewByText: function(text, allowInvisible, dontThrowIfNotFound, currentWindow) {
        LOGGER.info("############################################## findViewByText: " + text);
        return this.findView(function(wnd) {
            return wnd.text === text;
        }, allowInvisible, dontThrowIfNotFound, currentWindow);
    },
    findViewByTitle: function(title, allowInvisible, dontThrowIfNotFound, currentWindow) {
        LOGGER.info("############################################## findViewByTitle: " + title);
        return this.findView(function(wnd) {
            return wnd.title === title;
        }, allowInvisible, dontThrowIfNotFound, currentWindow);
    },
    findViewByValue: function(value, allowInvisible, dontThrowIfNotFound, currentWindow) {
        LOGGER.info("############################################## findViewByValue: " + value);
        return this.findView(function(wnd) {
            return wnd.value === value;
        }, allowInvisible, dontThrowIfNotFound, currentWindow);
    },
    waitForView: function(match, timeout, allowInvisible, currentWindow) {
        var dfd = Q.defer();
        timeout || (timeout = 2e3);
        var timeStarted = Date.now();
        var self = this;
        var check = function() {
            var wnd = self.findView(match, allowInvisible, true, currentWindow);
            if (wnd) dfd.resolve(wnd); else if (Date.now() < timeStarted + timeout) setTimeout(check, 50); else {
                self.dumpWindows(currentWindow);
                dfd.reject(new Error("Could not find window after " + timeout + "ms [allowInvisible: " + !!allowInvisible + "]"));
            }
        };
        check();
        return dfd.promise;
    },
    waitForAnyWindow: function(matches, timeout, allowInvisible, currentWindow) {
        LOGGER.info("############################################## waitForAnyWindow");
        var dfd = Q.defer();
        timeout || (timeout = 2e3);
        var timeStarted = Date.now();
        var self = this;
        var check = function() {
            var wnds = [];
            var foundOne = false;
            matches.forEach(function(match) {
                var wnd = self.findView(match, allowInvisible, true, currentWindow);
                wnds.push(wnd);
                wnd && (foundOne = true);
            });
            if (foundOne) dfd.resolve(wnds); else if (Date.now() < timeStarted + timeout) setTimeout(check, 50); else {
                self.dumpWindows(currentWindow);
                dfd.reject(new Error("Could not find window after " + timeout + "ms [allowInvisible: " + !!allowInvisible + "]"));
            }
        };
        check();
        return dfd.promise;
    },
    waitForViewById: function(id, timeout, allowInvisible, currentWindow) {
        LOGGER.info("############################################## waitForViewById: " + id);
        return this.waitForView(function(wnd) {
            return wnd.id === id;
        }, timeout, allowInvisible, currentWindow);
    },
    waitForViewByText: function(text, timeout, allowInvisible, currentWindow) {
        LOGGER.info("############################################## waitForViewByText: " + text);
        return this.waitForView(function(wnd) {
            return wnd.text === text;
        }, timeout, allowInvisible, currentWindow);
    },
    waitForViewByTitle: function(title, timeout, allowInvisible, currentWindow) {
        LOGGER.info("############################################## waitForViewByTitle: " + title);
        return this.waitForView(function(wnd) {
            return wnd.title === title;
        }, timeout, allowInvisible, currentWindow);
    },
    waitForViewByValue: function(value, timeout, allowInvisible, currentWindow) {
        LOGGER.info("############################################## waitForViewByValue: " + value);
        return this.waitForView(function(wnd) {
            return wnd.value === value;
        }, timeout, allowInvisible, currentWindow);
    },
    waitUntilTrue: function(test, timeout) {
        LOGGER.info("############################################## waitUntilTrue");
        var dfd = Q.defer();
        timeout || (timeout = 2e3);
        var timeStarted = Date.now();
        var check = function() {
            var result = test();
            result ? dfd.resolve(result) : Date.now() < timeStarted + timeout ? setTimeout(check, 50) : dfd.reject(new Error("Test function still failing after " + timeout + "ms"));
        };
        check();
        return dfd.promise;
    }
};

initialize();