"use strict";

var avatarUtil = require("ui/common/Avatar");

var Globals = require("/v6/infrastructure/Globals"), LoggingService = Globals.IOC.get("LoggingService");

var _ONLINE_THRESHOLD = 18e5, _RECENT_ONLINE_THRESHOLD = 72e5, _COLOR_OFFLINE = "#bebebe", _COLOR_RECENT_ONLINE = "#f4dd5e", _COLOR_ONLINE = "#b1ec11";

var CACHE_FOLDER = Ti.Filesystem.getFile(Ti.Filesystem.applicationCacheDirectory, "profile-cache");

CACHE_FOLDER.exists() || CACHE_FOLDER.createDirectory();

var localFileName = function(imageURL) {
    return Ti.Utils.md5HexDigest(imageURL + "");
};

var getStatusColor = function(lastSeen) {
    if (!lastSeen) return _COLOR_OFFLINE;
    var diff = Date.now() - lastSeen;
    if (_ONLINE_THRESHOLD > diff) return _COLOR_ONLINE;
    if (_RECENT_ONLINE_THRESHOLD > diff) return _COLOR_RECENT_ONLINE;
    return _COLOR_OFFLINE;
};

var Profile = function(args, user) {
    var defaultImage = args.defaultImage ? args.defaultImage : avatarUtil.getDefaultPhoto(user), radius = args.width ? parseInt(parseInt(args.width) / 2) : 20, imageViewParams = {
        top: "0",
        width: args.width ? args.width : "40dp",
        height: args.height ? args.height : "40dp",
        borderRadius: Alloy.Globals.Style.Unit["unit" + (_.isNumber(radius) ? radius : "20")],
        borderColor: "#fff",
        borderWidth: 0,
        defaultImage: defaultImage
    };
    args.image && _.extend(imageViewParams, {
        image: args.image
    });
    var padding = "2dp";
    if (args.padding) {
        padding = args.padding;
        delete args.padding;
    }
    var getPadding = function(_align, _padding) {
        var align = {
            left: _padding,
            right: null
        };
        switch (_align) {
          case "left":
            align = {
                left: _padding,
                right: null
            };
            break;

          case "right":
            align = {
                right: _padding,
                left: null
            };
        }
        return align;
    };
    if (args.align) {
        imageViewParams = _.extend(imageViewParams, getPadding(args.align, padding));
        delete args.align;
    }
    var viewParams = _.extend({
        width: "48dp",
        height: "48dp"
    }, args);
    var view = Ti.UI.createView(viewParams), imageView = Ti.UI.createImageView(imageViewParams);
    var statusView = Ti.UI.createView({
        bottom: "0dp",
        left: "0dp",
        width: "10dp",
        height: "10dp",
        borderRadius: Alloy.Globals.Style.Unit.unit5,
        borderColor: "#fff",
        borderWidth: ".5dp",
        zIndex: 99
    });
    view.add(imageView);
    view.add(statusView);
    view.imageView = imageView;
    view.statusView = statusView;
    var setUser = function(user) {
        if (user.hideStatus) view.remove(statusView); else {
            var lastSeen = user.lastSeen || user.lastKnownLocation && user.lastKnownLocation.timestamp;
            statusView.backgroundColor = getStatusColor(lastSeen);
        }
        loadUserAvatar(user, imageView);
        var handleProfileClick = function() {
            if (view.touchEnabled) {
                view.touchEnabled = false;
                (imageView.user.userId || imageView.user.id) && Ti.App.fireEvent("carma:profile.open", {
                    userId: imageView.user.userId || imageView.user.id
                });
                setTimeout(function() {
                    view.touchEnabled = true;
                }, 600);
            }
        };
        if (user.disableProfile) {
            view.removeEventListener("click", handleProfileClick);
            view.touchEnabled = false;
        } else {
            view.touchEnabled = true;
            imageView.user = user;
            view.addEventListener("click", handleProfileClick);
        }
        setImageViewSize(imageView, user.layout);
    };
    view.setUser = setUser;
    if (user) setUser(user); else {
        view.remove(statusView);
        var _properties = {
            defaultImage: defaultImage
        };
        args.image && _.extend(_properties, {
            image: args.image
        });
        imageView.applyProperties(_properties);
    }
    var highlightUser = function(color) {
        color = color || Alloy.Globals.Style.Color.LightBlue;
        imageView.setBorderColor(color);
        var ticker = Ti.UI.createLabel({
            font: {
                fontSize: "15dp",
                fontFamily: "splinico"
            },
            text: "c",
            color: "#fff",
            backgroundColor: color,
            borderRadius: Alloy.Globals.Style.Unit.unit9,
            borderWidth: 1,
            borderColor: color,
            textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
            width: "18dp",
            height: "18dp",
            bottom: "1dp",
            right: "1dp"
        });
        view.add(ticker);
    };
    view.highlightUser = highlightUser;
    view.setPadding = function(pros) {
        pros && pros.align && pros.padding && imageView.applyProperties(getPadding(pros.align, pros.padding));
    };
    return view;
};

var setImageViewSize = function(imageView, layout) {
    "circular-tiny" === layout ? imageView.applyProperties({
        top: "2dp",
        width: "32dp",
        height: "32dp",
        borderRadius: Alloy.Globals.Style.Unit.unit16,
        borderWidth: 1
    }) : "circular-small" === layout ? imageView.applyProperties({
        width: "48dp",
        height: "48dp",
        borderRadius: Alloy.Globals.Style.Unit.unit24,
        borderWidth: 1
    }) : "circular" === layout ? imageView.applyProperties({
        top: "2dp",
        width: "60dp",
        height: "60dp",
        borderRadius: Alloy.Globals.Style.Unit.unit30,
        borderWidth: 1
    }) : "circular-large" === layout && imageView.applyProperties({
        top: "2dp",
        width: "64dp",
        height: "64dp",
        borderRadius: Alloy.Globals.Style.Unit.unit32,
        borderWidth: 1
    });
};

var loadUserAvatar = function(user, imageView) {
    if (!user || !user.photoUrl) {
        imageView.applyProperties({
            defaultImage: getDefaultUserAvatar(user),
            image: getDefaultUserAvatar(user)
        });
        return;
    }
    var size = getAvatarSize(user), url = require("utils/photoURL").toURL(user.photoUrl, {
        width: size,
        height: size
    }), file = Ti.Filesystem.getFile(CACHE_FOLDER.resolve(), localFileName(url));
    if (file.exists()) {
        imageView.applyProperties({
            image: file.getNativePath()
        });
        return;
    }
    imageView.applyProperties({
        image: getDefaultUserAvatar(user)
    });
    var client = Ti.Network.createHTTPClient({
        onload: function() {
            file.write(this.responseData);
            imageView.applyProperties({
                image: this.responseData
            });
        },
        timeout: 5e3
    });
    client.open("GET", url);
    LoggingService.log("network", url);
    client.send();
};

var getAvatarSize = function(user) {
    if (!user || !user.layout) return 60;
    if ("circular-tiny" === user.layout) return 32;
    if ("circular-small" === user.layout) return 50;
    if ("circular" === user.layout) return 60;
    if ("circular-large" === user.layout) return 64;
    return 60;
};

var getDefaultUserAvatar = function(user) {
    return avatarUtil.getDefaultPhoto(user);
};

exports.createView = function(args, user) {
    !user && "user" in args && (user = args.user);
    return new Profile(args, user);
};