"use strict";

var _ = require("/vendor/underscore"), Q = require("/vendor/q"), PeriodicService = require("/v6/common/PeriodicService"), PendingDataAPICall = require("/v6/network/entity/PendingDataAPICall"), BoardingAPICompiler = require("./helpers/BoardingAPICompiler"), APICallResponseStrategy = require("/v6/network/strategy/APICallResponseStrategy"), CurrentStatusService = require("./CurrentStatusService"), BoardingAPIResponseStrategies = require("./strategy/BoardingApiResponseStrategies"), TimeValidation = require("/v6/common/validation/TimeValidation");

var BoardingBackgroundService = function(currentUser, boarding, summary, network, loggingService, uiMessenger, profileService) {
    if (!currentUser) throw new Error("CurrentUserService is required");
    if (!boarding) throw new Error("BoardingService is required");
    if (!summary) throw new Error("SummaryService is required");
    if (!network) throw new Error("NetworkService is required");
    if (!loggingService) throw new Error("LoggingService is required");
    if (!uiMessenger) throw new Error("UIMessengerService is required");
    if (!profileService) throw new Error("ProfileService is required");
    var logger = loggingService.logger("boarding");
    var strategy = new APICallResponseStrategy(new BoardingAPIResponseStrategies(currentUser, boarding, summary, logger, uiMessenger, profileService), logger);
    var currentStatusService = new CurrentStatusService(network);
    var calls = function() {
        var drives = boarding.getDrives();
        var apiCalls = BoardingAPICompiler.compile(currentUser, drives);
        var validCalls = [], pendingAPICall = null;
        _.each(apiCalls, function(apiCall) {
            if (apiCall instanceof PendingDataAPICall) {
                pendingAPICall = apiCall;
                return false;
            }
            validCalls.push(apiCall);
        });
        logger.log(_.template("There are currently <%= ndrives %> drives", {
            ndrives: drives.length
        }));
        pendingAPICall && logger.log(_.template("APICall is waiting for data '<%= type %>'", {
            type: pendingAPICall.type
        }));
        return validCalls;
    };
    var execOfflineAPICall = function(apiCall) {
        var dfd = Q.defer();
        logger.log(_.template("ApiCall <%= type %>", {
            type: apiCall.type
        }));
        network.request(apiCall).then(function(data) {
            try {
                var shouldContinue = strategy.execute(apiCall, data);
                dfd.resolve(shouldContinue);
            } catch (e) {
                dfd.reject(e);
            }
        }).catch(function(error) {
            currentStatusService.reset();
            try {
                var shouldContinue = strategy.execute(apiCall, error);
                dfd.resolve(shouldContinue);
            } catch (e) {
                dfd.reject(e);
            }
        });
        return dfd.promise;
    };
    var syncClientToServer = function() {
        var apiCalls = calls();
        if (!apiCalls.length) return Q(false);
        var apiCall = apiCalls.shift();
        return execOfflineAPICall(apiCall).then(function(shouldContinue) {
            return false === shouldContinue ? false : true;
        }).catch(function() {
            currentStatusService.reset();
            return false;
        });
    };
    var syncServerToClient = function() {
        var lastUpdatedOn = currentStatusService.lastUpdatedOn();
        logger.log(_.template("tick.getServerStatus(since=<%= since %>)", {
            since: lastUpdatedOn
        }));
        return currentStatusService.getServerStatus().then(function(data) {
            if (data) {
                logger.log(_.template("Things changed since=<%= since %>! Need to sync!", {
                    since: lastUpdatedOn
                }));
                return strategy.execute(data.apiCall, data.current);
            }
            logger.log(_.template("Nothing happened since=<%= since %>!", {
                since: lastUpdatedOn
            }));
        });
    };
    var tick = function(recur) {
        if (!currentUser.id) return;
        recur || logger.log("tick()");
        return Q().then(function() {
            return syncClientToServer();
        }).then(function(shouldTickAgain) {
            if (shouldTickAgain) return tick(true);
        }).then(function() {
            if (!recur) return syncServerToClient();
        }).catch(function(e) {
            logger.log("tick() ERROR");
            logger.log(e);
            logger.log(e.backtrace);
            throw e;
        });
    };
    var resetAllState = function() {
        currentStatusService.reset();
    };
    return _.extend(new PeriodicService(), {
        resetAllState: resetAllState,
        tick: tick,
        calls: calls
    });
};

BoardingBackgroundService.$inject = [ "CurrentUserService", "BoardingService", "SummaryService", "NetworkService", "LoggingService", "UIMessengerService", "ProfileService" ];

BoardingBackgroundService.$isRevealingModule = true;

module.exports = BoardingBackgroundService;