"use strict";

var _ = require("/vendor/underscore"), Validation = require("/v6/boarding/helpers/Validation"), BoardingInfo = require("./models/BoardingInfo"), BoardingInfoCollection = require("./models/BoardingInfoCollection"), Drive = require("./models/Drive"), DriveCollection = require("./models/DriveCollection"), Ride = require("./models/Ride");

var InvalidArguments = require("../common/errors/InvalidArguments"), DriveNotFound = require("./errors/DriveNotFound"), RideNotFound = require("./errors/RideNotFound"), AlreadyInCarError = require("./errors/AlreadyInCarError"), InvalidDriverError = require("./errors/InvalidDriverError"), NoCurrentDriveError = require("./errors/NoCurrentDriveError");

var Internal = {};

Internal.isUserInDrive = function(userId, drive) {
    if (!drive) return false;
    if (drive.get("driverId") == userId) return true;
    var userRides = drive.get("rides").where({
        riderId: userId
    });
    var isInDrive = false;
    _.each(userRides, function(ride, index) {
        if (!ride.get("state").get("done")) {
            isInDrive = true;
            return false;
        }
    });
    return isInDrive;
};

var BoardingService = function(currentUserService, loggingService, beaconsHelper, geofenceHelper) {
    if (!currentUserService) throw new Error("CurrentUserService is required");
    if (!loggingService) throw new Error("LoggingService is required");
    var logger = loggingService.logger("boarding");
    var getBoardingInfo = function() {
        var info = new BoardingInfoCollection().fetch();
        if (!info.length) return new BoardingInfo().save();
        return info.shift();
    };
    var self = {
        getCurrentDrive: function() {
            var info = getBoardingInfo();
            var drive = info.current();
            if (drive) return drive.toJSON();
            return drive;
        },
        startDrive: function(options) {
            options = _.extend({}, options, {
                driverId: options.driverId || 0,
                timestamp: options.timestamp || 0,
                lat: null == options.lat ? null : options.lat,
                lon: null == options.lon ? null : options.lon,
                accuracy: null == options.accuracy ? null : options.accuracy
            });
            var currentDrive = getBoardingInfo().get("current");
            if (currentDrive) throw new AlreadyInCarError();
            currentDrive = new Drive({
                driverId: options.driverId,
                startTimestamp: options.timestamp,
                startLat: options.lat,
                startLon: options.lon,
                startAccuracy: options.accuracy,
                rides: []
            }).save();
            geofenceHelper && options.driverId && geofenceHelper.startLocationMonitoring(options.driverId);
            getBoardingInfo().set("current", currentDrive).save();
            return currentDrive.toJSON();
        },
        joinCar: function(options) {
            options = _.extend({}, options, {
                rideId: options.rideId || null,
                riderId: options.riderId || 0,
                driverId: options.driverId || 0,
                driverPhoneNumber: options.driverPhoneNumber || null,
                driverBeaconId: options.driverBeaconId || null,
                timestamp: options.timestamp || 0,
                lat: null == options.lat ? null : options.lat,
                lon: null == options.lon ? null : options.lon,
                accuracy: null == options.accuracy ? null : options.accuracy,
                freeRide: options.freeRide
            });
            if (!options.riderId) throw new InvalidArguments("riderId");
            var currentDrive = getBoardingInfo().get("current");
            if (currentDrive) {
                if (currentDrive.get("driverId") != options.driverId) throw new InvalidDriverError();
            } else currentDrive = new Drive({
                driverId: options.driverId,
                driverPhoneNumber: options.driverPhoneNumber,
                driverBeaconId: options.driverBeaconId,
                startTimestamp: options.timestamp,
                rides: []
            });
            if (Internal.isUserInDrive(options.riderId, currentDrive)) throw new AlreadyInCarError();
            var ride = new Ride({
                rideId: options.rideId,
                riderId: options.riderId,
                driverId: options.driverId,
                driverPhoneNumber: options.driverPhoneNumber,
                driverBeaconId: options.driverBeaconId,
                startTimestamp: options.timestamp,
                startLat: options.lat,
                startLon: options.lon,
                startAccuracy: options.accuracy,
                freeRide: options.freeRide
            }).save();
            currentDrive.get("rides").push(ride);
            currentDrive.save();
            getBoardingInfo().get("current") || getBoardingInfo().set("current", currentDrive).save();
            return currentDrive.toJSON();
        },
        endRide: function(options) {
            options = _.extend({}, options, {
                riderId: options.riderId,
                driverId: options.driverId || 0,
                driverBeaconId: options.driverBeaconId || null,
                timestamp: options.timestamp || 0,
                lat: null == options.lat ? null : options.lat,
                lon: null == options.lon ? null : options.lon,
                accuracy: null == options.accuracy ? null : options.accuracy
            });
            if (!options.riderId) throw new InvalidArguments("riderId");
            var currentDrive = getBoardingInfo().get("current");
            if (!currentDrive) throw new NoCurrentDriveError();
            if (options.driverBeaconId && currentDrive.get("driverBeaconId") != options.driverBeaconId) throw new InvalidDriverError();
            if (!currentDrive.get("driverPhoneNumber") && currentDrive.get("driverId") != options.driverId) throw new InvalidDriverError();
            var ride = currentDrive.get("rides").where({
                riderId: options.riderId
            }).shift();
            if (!ride) throw new RideNotFound();
            ride.set({
                endTimestamp: options.timestamp,
                endLat: options.lat,
                endLon: options.lon,
                endAccuracy: options.accuracy
            }, {
                silent: true
            }).save();
            currentDrive.get("driverId") != currentUserService.id && getBoardingInfo().set("current", null).save();
            return currentDrive.toJSON();
        },
        leaveCar: function(options) {
            return self.endRide(options);
        },
        dropOff: function(options) {
            return self.endRide(options);
        },
        endDrive: function(options) {
            options = _.extend({}, options, {
                driverId: options.driverId || 0,
                timestamp: options.timestamp || 0,
                lat: null == options.lat ? null : options.lat,
                lon: null == options.lon ? null : options.lon,
                accuracy: null == options.accuracy ? null : options.accuracy
            });
            var currentDrive = getBoardingInfo().get("current");
            if (!currentDrive) throw new NoCurrentDriveError();
            if (currentDrive.get("driverId") != options.driverId) throw new InvalidDriverError();
            currentDrive.set({
                endTimestamp: options.timestamp,
                endLat: options.lat,
                endLon: options.lon,
                endAccuracy: options.accuracy
            }, {
                silent: true
            }).save();
            geofenceHelper && options.driveId && geofenceHelper.stopLocationMonitoring(options.driverId);
            getBoardingInfo().set("current", null).save();
            return currentDrive.toJSON();
        },
        isDriveSummaryReady: function() {
            throw new Error("Not implemented yet!");
        },
        deleteDrive: function(id) {
            var drive = new Drive().fetch({
                id: id
            });
            if (!drive) throw new DriveNotFound();
            drive.destroy();
        },
        getDrive: function(id) {
            var drive = new Drive().fetch({
                id: id
            });
            if (drive) return drive.toJSON();
            return null;
        },
        getDrives: function() {
            var drives = new DriveCollection().fetch();
            return drives.toJSON();
        },
        getDrivesWithMissingGPS: function() {
            var drives = new DriveCollection().fetch(), result = new DriveCollection();
            _.each(drives, function(drive, index) {
                drive = drives.at(index);
                var ridesNeedingGPS = [], isMyDrive = drive.get("driverId") == currentUserService.id;
                _.each(drive.get("rides"), function(ride, index) {
                    ride = drive.get("rides").at(index);
                    ride.needsGPS() && ridesNeedingGPS.push(ride);
                });
                if (drive.needsGPS() && isMyDrive || ridesNeedingGPS.length > 0) {
                    drive.get("rides").reset(ridesNeedingGPS);
                    result.push(drive);
                }
            });
            return result.toJSON();
        },
        updateDriveGPS: function(id, data) {
            if (!id) throw new InvalidArguments("id");
            data = data || {};
            data.driveId = data.driveId || null;
            data.driverId = data.driverId || null;
            data.startLat = null == data.startLat ? null : data.startLat;
            data.startLon = null == data.startLon ? null : data.startLon;
            data.startAccuracy = null == data.startAccuracy ? null : data.startAccuracy;
            data.endLat = null == data.endLat ? null : data.endLat;
            data.endLon = null == data.endLon ? null : data.endLon;
            data.endAccuracy = null == data.endAccuracy ? null : data.endAccuracy;
            var drive = new Drive().fetch({
                id: id
            });
            if (!drive) throw new DriveNotFound();
            var fieldsToUpdate = {};
            drive.needStartGPS() && Validation.isValidGPSPosition(data.startLat) && _.extend(fieldsToUpdate, {
                startLat: data.startLat,
                startLon: data.startLon,
                startAccuracy: data.startAccuracy
            });
            drive.needEndGPS() && Validation.isValidGPSPosition(data.endLat) && _.extend(fieldsToUpdate, {
                endLat: data.endLat,
                endLon: data.endLon,
                endAccuracy: data.endAccuracy
            });
            if (data.driveId) {
                fieldsToUpdate.driveId = data.driveId;
                beaconsHelper && beaconsHelper.setupActiveTrip({
                    id: data.driveId
                }, true);
            }
            data.driverId && (fieldsToUpdate.driverId = data.driverId);
            if (_.keys(fieldsToUpdate).length) {
                drive.set(fieldsToUpdate, {
                    silent: true
                });
                drive.save();
            }
            return drive.toJSON();
        },
        updateRideGPS: function(id, data) {
            if (!id) throw new InvalidArguments("id");
            data = data || {};
            data.rideId = data.rideId || null;
            data.startLat = null == data.startLat ? null : data.startLat;
            data.startLon = null == data.startLon ? null : data.startLon;
            data.startAccuracy = null == data.startAccuracy ? null : data.startAccuracy;
            data.endLat = null == data.endLat ? null : data.endLat;
            data.endLon = null == data.endLon ? null : data.endLon;
            data.endAccuracy = null == data.endAccuracy ? null : data.endAccuracy;
            data.freeRide = null == data.freeRide ? null : !!data.freeRide;
            var ride = new Ride().fetch({
                id: id
            });
            if (!ride) throw new RideNotFound();
            var toUpdate = {};
            ride.needStartGPS() && Validation.isValidGPSPosition(data.startLat) && _.extend(toUpdate, {
                startLat: data.startLat,
                startLon: data.startLon,
                startAccuracy: data.startAccuracy
            });
            ride.needEndGPS() && Validation.isValidGPSPosition(data.endLat) && _.extend(toUpdate, {
                endLat: data.endLat,
                endLon: data.endLon,
                endAccuracy: data.endAccuracy
            });
            if (data.rideId) {
                _.extend(toUpdate, {
                    rideId: data.rideId,
                    driverFirstName: data.driverFirstName,
                    driverPhotoUrl: data.driverPhotoUrl,
                    driverAlias: data.driverAlias
                });
                beaconsHelper && beaconsHelper.setupActiveTrip({
                    journeyId: data.rideId,
                    driverId: ride.get("driverId"),
                    driverFirstName: data.driverFirstName,
                    driverAlias: data.driverAlias,
                    driverPhotoUrl: data.driverPhotoUrl
                }, false);
            }
            data.freeRide && _.extend(toUpdate, {
                freeRide: data.freeRide
            });
            ride.set(toUpdate, {
                silent: true
            });
            ride.save();
            return ride.toJSON();
        },
        updateDriveState: function(driveId, state) {
            state = state || {};
            state.done = null != state.done ? state.done : false;
            state.error = null != state.error ? state.error : false;
            state.errorMessage = state.errorMessage || null;
            state.errorCode = state.errorCode || null;
            var drive = new Drive().fetch({
                id: driveId
            });
            if (null == drive) return;
            var dstate = drive.get("state");
            dstate.set({
                done: state.done,
                error: state.error,
                errorMessage: state.errorMessage,
                errorCode: state.errorCode
            }).save();
            if (dstate.get("done")) {
                var info = getBoardingInfo();
                info.get("current") && info.get("current").get("id") == drive.get("id") && info.set("current", null).save();
            }
            return drive.toJSON();
        },
        updateRideState: function(rideId, state) {
            state = state || {};
            state.done = null != state.done ? state.done : false;
            state.error = null != state.error ? state.error : false;
            state.errorMessage = state.errorMessage || null;
            state.errorCode = state.errorCode || null;
            var ride = new Ride().fetch({
                id: rideId
            });
            if (null == ride) return;
            var dstate = ride.get("state");
            dstate.set({
                done: state.done,
                error: state.error,
                errorMessage: state.errorMessage,
                errorCode: state.errorCode
            }).save();
            return ride.toJSON();
        },
        resetAllState: function() {
            var drives = new DriveCollection().fetch();
            for (var i = 0; i < drives.length; i++) {
                var drive = drives.at(i);
                drive.destroy();
            }
        },
        popCorruptedDrives: function() {
            throw new Error("Not implemented yet!");
        }
    };
    "function" == typeof logger.logDuration && _.each(self, function(fn, name) {
        self[name] = logger.logDuration(fn, {
            name: name,
            scope: self
        });
    });
    return self;
};

BoardingService.$inject = [ "CurrentUserService", "LoggingService", "beaconsHelper", "geofenceHelper" ];

BoardingService.$isRevealingModule = true;

module.exports = BoardingService;