"use strict";

var Q = require("/vendor/q"), _ = require("/vendor/underscore"), Summary = require("./models/Summary"), SummaryCollection = require("./models/SummaryCollection"), SummaryAchievement = require("./models/SummaryAchievement"), SummaryAchievementCollection = require("./models/SummaryAchievementCollection"), SummaryEarning = require("./models/SummaryEarning"), SummaryRide = require("./models/SummaryRide"), SummaryRideCollection = require("./models/SummaryRideCollection"), SummaryReward = require("./models/SummaryReward"), SummaryRewardCollection = require("./models/SummaryRewardCollection"), SummaryRewardTotalCollection = require("./models/SummaryRewardTotalCollection"), SummaryRewardTotal = require("./models/SummaryRewardTotal"), APICall = require("v6/network/entity/APICall"), APICallType = require("v6/network/entity/APICallType"), BoardingLogMessages = require("/v6/boarding/helpers/BoardingLogMessages");

var DriveNotFound = require("./errors/DriveNotFound"), SummaryNotFound = require("./errors/SummaryNotFound");

var SummaryService = function(currentUserService, boardingService, networkService, loggingService) {
    if (!currentUserService) throw new Error("CurrentUserService is required");
    if (!boardingService) throw new Error("BoardingService is required");
    if (!networkService) throw new Error("NetworkService is required");
    if (!loggingService) throw new Error("LoggingService is required");
    var logger = loggingService.logger("boarding");
    var isSummaryDriveCompleted = function(summaryModel) {
        return !!summaryModel.get("drive").get("state").get("done");
    };
    var getLocalDriveSummaryModel = function(driveId) {
        var summaries = new SummaryCollection().fetch(), result = null;
        _.each(summaries, function(summary, index) {
            summary = summaries.at(index);
            if (summary.get("drive") && summary.get("drive").get("id") == driveId) {
                result = summary;
                return false;
            }
        });
        return result;
    };
    var updateSummary = function(id, data) {
        var summaryModel = new Summary().fetch({
            id: id
        });
        var driveError = summaryModel.get("drive").get("state").get("error");
        var isError = driveError || !!data.unrealisticCause && data.rides.length > 0, isPendingData = !driveError && !data.unrealisticCause && null == data.distance;
        var toUpdate = {
            distance: data.distance,
            startTimestamp: data.startTimestamp,
            startAddress: data.startAddress,
            endTimestamp: data.endTimestamp,
            endAddress: data.endAddress,
            unrealisticCause: data.unrealisticCause || "",
            isError: isError,
            isPendingData: isPendingData,
            rewards: new SummaryRewardCollection(),
            achievements: new SummaryAchievementCollection(),
            rides: new SummaryRideCollection(),
            rewardTotals: new SummaryRewardTotalCollection()
        };
        data.earning && (toUpdate.earning = new SummaryEarning(data.earning).save());
        _.each(data.rewardTotals, function(rewardTotal) {
            toUpdate.rewardTotals.push(new SummaryRewardTotal(rewardTotal).save());
        });
        _.each(data.achievements, function(achievement) {
            toUpdate.achievements.push(new SummaryAchievement(achievement).save());
        });
        _.each(data.rides, function(ride) {
            var rideModel = new SummaryRide(ride).save();
            toUpdate.rides.push(rideModel);
        });
        _.each(data.rewards, function(reward) {
            toUpdate.rewards.push(new SummaryReward(reward).save());
        });
        summaryModel = summaryModel.set(toUpdate).save();
        return summaryModel.toJSON();
    };
    var fetchSummaryFromNetwork = function(summary) {
        var ApiType = currentUserService.id == summary.drive.driverId ? APICallType.DriveHistory : APICallType.RideHistory;
        var apiCall = new APICall(ApiType, Date.now(), summary);
        return networkService.request(apiCall).then(function(data) {
            switch (ApiType) {
              case APICallType.RideHistory:
                _.extend(data, {
                    rides: [ {
                        driveId: summary.drive.id,
                        riderId: currentUserService.id,
                        duration: data.startTimestamp - data.endTimestamp,
                        freeRide: data.freeRide
                    } ]
                });
            }
            return data;
        });
    };
    var fetchSummary = function(summaryId) {
        var summary = new Summary().fetch({
            id: summaryId
        });
        if (summary) {
            var isError = summary.get("isError"), isPendingData = summary.get("isPendingData"), driveCompleted = isSummaryDriveCompleted(summary);
            if (isPendingData && driveCompleted) {
                logger.log("SummaryService.fetchSummaryFromNetwork()");
                logger.log(function() {
                    var drive = summary.get("drive").toJSON();
                    return BoardingLogMessages.driveStatusLogMessage(drive);
                });
                return fetchSummaryFromNetwork(summary.toJSON()).then(function(data) {
                    return updateSummary(summaryId, data);
                }).catch(function(e) {
                    logger.log("SummaryService.fetchSummaryFromNetwork() Error");
                    e.isOffline || summary.set("isError", true).save();
                    return summary.toJSON();
                });
            }
            return Q(summary.toJSON());
        }
        return Q(null);
    };
    var self = {
        saveSummary: function(options) {
            options = _.extend({}, options, {
                driveId: options.driveId || 0,
                distance: null == options.distance ? null : options.distance,
                startTimestamp: null == options.startTimestamp ? null : options.startTimestamp,
                startAddress: null == options.startAddress ? null : options.startAddress,
                endTimestamp: null == options.endTimestamp ? null : options.endTimestamp,
                endAddress: null == options.endAddress ? null : options.endAddress,
                earning: options.earning ? options.earning : null,
                achievements: options.achievements ? options.achievements : [],
                rides: options.rides ? options.rides : [],
                rewards: options.rewards ? options.rewards : [],
                rewardTotals: options.rewardTotals ? options.rewardTotals : null,
                unrealisticCause: options.unrealisticCause || ""
            });
            var drive = boardingService.getDrive(options.driveId);
            if (!drive) throw new DriveNotFound();
            var summary = getLocalDriveSummaryModel(options.driveId);
            if (!summary) {
                var isError = !!drive.state.error;
                summary = new Summary({
                    drive: drive,
                    isError: isError,
                    isPendingData: !isError
                }).save();
            }
            return updateSummary(summary.get("id"), options);
        },
        getSummary: function(summaryId) {
            return fetchSummary(summaryId);
        },
        getDriveSummaryReference: function(driveId) {
            var drive = boardingService.getDrive(driveId), summaryModel = getLocalDriveSummaryModel(driveId);
            if (drive && !summaryModel) {
                var isError = !!drive.state.error;
                summaryModel = new Summary({
                    drive: drive,
                    isError: isError,
                    isPendingData: !isError
                }).save();
            }
            return summaryModel.get("id");
        },
        getSummariesReferences: function() {
            var drives = boardingService.getDrives(), completed = [];
            _.each(drives, function(drive, index) {
                if (drive.state.done) {
                    var sumId = self.getDriveSummaryReference(drive.id);
                    completed.push(sumId);
                }
            });
            return completed;
        },
        deleteSummary: function(summaryId) {
            var summary = new Summary().fetch({
                id: summaryId
            });
            if (!summary) throw new SummaryNotFound();
            logger.log(function() {
                var summ = summary.toJSON();
                return _.template("SummaryService.deleteSummary(driveId=<%= driveId %>)", _.extend({
                    driveId: null
                }, summ.drive));
            });
            summary.get("drive").destroy();
            summary.destroy();
        },
        dispose: function() {}
    };
    "function" == typeof logger.logDuration && _.each(self, function(fn, name) {
        self[name] = logger.logDuration(fn, {
            name: name,
            scope: self
        });
    });
    getLocalDriveSummaryModel = logger.logDuration(getLocalDriveSummaryModel, {
        name: "getLocalDriveSummaryModel"
    });
    updateSummary = logger.logDuration(updateSummary, {
        name: "updateSummary"
    });
    return self;
};

SummaryService.$inject = [ "CurrentUserService", "BoardingService", "NetworkService", "LoggingService" ];

SummaryService.$isRevealingModule = true;

module.exports = SummaryService;