"use strict";

var InvalidArgument = require("/v6/common/errors/InvalidArguments"), TimestampHelper = require("/v6/boarding/helpers/TimestampHelper"), Validation = require("/v6/boarding/helpers/Validation"), Q = require("/vendor/q"), errorCodes = require("services/error_codes");

var QUESTIONS = {
    JOIN_CAR: 1,
    LEAVE_CAR: 2,
    PICK_UP: 3,
    DROP_OFF: 4,
    START_SOLO: 5,
    END_SOLO: 6
};

var createDriverStartQuestion = function(driveToProcess, timestamp) {
    var buddies = [];
    driveToProcess.rides.forEach(function(ride) {
        !Validation.isValidGPSPosition(ride.startLat) && TimestampHelper.isSingleEvent(timestamp, ride.startTimestamp) && buddies.push({
            id: ride.riderId
        });
    });
    return {
        question: buddies.length > 0 ? QUESTIONS.PICK_UP : QUESTIONS.START_SOLO,
        errorCode: errorCodes.ERROR_SOURCE_BOARDING,
        timestamp: timestamp,
        buddies: buddies
    };
};

var createDriverEndQuestion = function(driveToProcess, timestamp, isEndDrive) {
    var buddies = [];
    driveToProcess.rides.forEach(function(ride) {
        !Validation.isValidGPSPosition(ride.endLat) && (TimestampHelper.isSingleEvent(timestamp, ride.endTimestamp) || !Validation.isValidTimestamp(ride.endTimestamp) && isEndDrive) && buddies.push({
            id: ride.riderId
        });
    });
    return {
        question: buddies.length > 0 ? QUESTIONS.DROP_OFF : QUESTIONS.END_SOLO,
        errorCode: errorCodes.ERROR_SOURCE_DROPOFF,
        timestamp: timestamp,
        buddies: buddies
    };
};

var compileForDrivers = function(driveToProcess) {
    if (!Validation.isValidGPSPosition(driveToProcess.startLat)) return createDriverStartQuestion(driveToProcess, driveToProcess.startTimestamp);
    var timestampFound = null;
    driveToProcess.rides.forEach(function(ride) {
        null !== timestampFound || Validation.isValidGPSPosition(ride.startLat) || (timestampFound = ride.startTimestamp);
    });
    if (timestampFound) return createDriverStartQuestion(driveToProcess, timestampFound);
    if (!Validation.isValidTimestamp(driveToProcess.endTimestamp) || Validation.isValidGPSPosition(driveToProcess.endLat)) {
        driveToProcess.rides.forEach(function(ride) {
            null === timestampFound && Validation.isValidTimestamp(ride.endTimestamp) && !Validation.isValidGPSPosition(ride.endLat) && (timestampFound = ride.endTimestamp);
        });
        if (timestampFound) return createDriverEndQuestion(driveToProcess, timestampFound, false);
    }
    if (Validation.isValidTimestamp(driveToProcess.endTimestamp) && !Validation.isValidGPSPosition(driveToProcess.endLat)) return createDriverEndQuestion(driveToProcess, driveToProcess.endTimestamp, true);
    throw new InvalidArgument("Drives supposedly containing missing GPS locations, but none found");
};

var compileForRiders = function(driveToProcess) {
    if (!Validation.isValidGPSPosition(driveToProcess.rides[0].startLat)) return {
        question: QUESTIONS.JOIN_CAR,
        errorCode: errorCodes.ERROR_SOURCE_BOARDING,
        timestamp: driveToProcess.rides[0].startTimestamp,
        buddies: [ {
            id: driveToProcess.driverId,
            driverPhoneNo: driveToProcess.driverPhoneNumber
        } ]
    };
    if (Validation.isValidTimestamp(driveToProcess.rides[0].endTimestamp) && !Validation.isValidGPSPosition(driveToProcess.rides[0].endLat)) return {
        question: QUESTIONS.LEAVE_CAR,
        errorCode: errorCodes.ERROR_SOURCE_DROPOFF,
        timestamp: driveToProcess.rides[0].endTimestamp,
        buddies: [ {
            id: driveToProcess.driverId,
            driverPhoneNo: driveToProcess.driverPhoneNumber
        } ]
    };
    throw new InvalidArgument("Drives supposedly containing missing GPS locations, but none found");
};

var createStartLocation = function(location) {
    return {
        startLat: location.latitude,
        startLon: location.longitude,
        startAccuracy: location.accuracy
    };
};

var createEndLocation = function(location) {
    return {
        endLat: location.latitude,
        endLon: location.longitude,
        endAccuracy: location.accuracy
    };
};

var BoardingMissingGPSCompiler = function(boardingService) {
    return {
        _QUESTIONS: QUESTIONS,
        compile: function(userId, driveToProcess) {
            var isDriver = userId == driveToProcess.driverId;
            return isDriver ? compileForDrivers(driveToProcess) : compileForRiders(driveToProcess);
        },
        findLocation: function(drivesToProcess, timestamp) {
            var delta = null;
            var location = null;
            drivesToProcess.forEach(function(drive) {
                if (Validation.isValidGPSPosition(drive.startLat) && (null === delta || Math.abs(drive.startTimestamp - timestamp) < delta)) {
                    delta = Math.abs(drive.startTimestamp - timestamp);
                    location = {
                        latitude: drive.startLat,
                        longitude: drive.startLon,
                        accuracy: drive.startAccuracy
                    };
                }
                if (Validation.isValidGPSPosition(drive.endLat) && (null === delta || Math.abs(drive.endTimestamp - timestamp) < delta)) {
                    delta = Math.abs(drive.endTimestamp - timestamp);
                    location = {
                        latitude: drive.endLat,
                        longitude: drive.endLon,
                        accuracy: drive.endAccuracy
                    };
                }
                drive.rides.forEach(function(ride) {
                    if (Validation.isValidGPSPosition(ride.startLat) && (null === delta || Math.abs(ride.startTimestamp - timestamp) < delta)) {
                        delta = Math.abs(ride.startTimestamp - timestamp);
                        location = {
                            latitude: ride.startLat,
                            longitude: ride.startLon,
                            accuracy: ride.startAccuracy
                        };
                    }
                    if (Validation.isValidGPSPosition(ride.endLat) && (null === delta || Math.abs(ride.endTimestamp - timestamp) < delta)) {
                        delta = Math.abs(ride.endTimestamp - timestamp);
                        location = {
                            latitude: ride.endLat,
                            longitude: ride.endLon,
                            accuracy: ride.endAccuracy
                        };
                    }
                });
            });
            return location && TimestampHelper.isSingleEvent(0, delta) ? location : null;
        },
        updateMissingLocations: function(drivesToProcess, timestamp, location) {
            if (!drivesToProcess || !timestamp || !location) throw new InvalidArgument("Invalid parameters");
            var somethingUpdated = false;
            drivesToProcess.forEach(function(drive) {
                if (!Validation.isValidGPSPosition(drive.startLat) && TimestampHelper.isSingleEvent(timestamp, drive.startTimestamp)) {
                    boardingService.updateDriveGPS(drive.id, createStartLocation(location));
                    somethingUpdated = true;
                }
                if (!Validation.isValidGPSPosition(drive.endLat) && TimestampHelper.isSingleEvent(timestamp, drive.endTimestamp)) {
                    boardingService.updateDriveGPS(drive.id, createEndLocation(location));
                    somethingUpdated = true;
                }
                drive.rides.forEach(function(ride) {
                    if (!Validation.isValidGPSPosition(ride.startLat) && TimestampHelper.isSingleEvent(timestamp, ride.startTimestamp)) {
                        boardingService.updateRideGPS(ride.id, createStartLocation(location));
                        somethingUpdated = true;
                    }
                    if (!Validation.isValidGPSPosition(ride.endLat) && TimestampHelper.isSingleEvent(timestamp, ride.endTimestamp)) {
                        boardingService.updateRideGPS(ride.id, createEndLocation(location));
                        somethingUpdated = true;
                    }
                });
            });
            return somethingUpdated;
        },
        handleMissingGPS: function(userId, drivesToProcess, manualLocationInput) {
            var deferred = Q.defer();
            var question = this.compile(userId, drivesToProcess[0]);
            var location = this.findLocation(drivesToProcess, question.timestamp);
            if (location) try {
                var somethingUpdated = this.updateMissingLocations(drivesToProcess, question.timestamp, location);
                deferred.resolve(somethingUpdated);
            } catch (e) {
                LOGGER.error("Could not update missing locations: " + JSON.stringify(e));
                deferred.resolve(false);
            } else {
                var self = this;
                Q().then(function() {
                    return manualLocationInput(drivesToProcess, question);
                }).then(function(location) {
                    if (location) try {
                        var somethingUpdated = self.updateMissingLocations(drivesToProcess, question.timestamp, location);
                        deferred.resolve(somethingUpdated);
                    } catch (e) {
                        LOGGER.error("Could not update missing locations: " + JSON.stringify(e));
                        deferred.resolve(false);
                    } else deferred.resolve(false);
                }).catch(function() {
                    deferred.resolve(false);
                });
            }
            return deferred.promise;
        }
    };
};

module.exports = BoardingMissingGPSCompiler;