"use strict";

var _ = require("/vendor/underscore"), BoardingLogMessages = require("/v6/boarding/helpers/BoardingLogMessages"), APICallType = require("/v6/network/entity/APICallType"), UIMessageTypes = require("/v6/uiMessenger/entity/UIMessageTypes");

var updateProfileCreditBalance = function(currentUserService, profileService, creditBalance) {
    creditBalance ? profileService.updateCredit(currentUserService.id, creditBalance.totalBalance, creditBalance.payableBalance, creditBalance.bonusBalance) : profileService.getProfile(currentUserService.id, profileService.FULL_DETAIL, true);
};

var BoardingAPIResponseStrategies = function(currentUser, boarding, summary, logger, uiMessenger, profileService) {
    if (!currentUser) throw new Error("CurrentUserService is required");
    if (!boarding) throw new Error("BoardingService is required");
    if (!summary) throw new Error("SummaryService is required");
    if (!logger) throw new Error("LoggingService is required");
    if (!uiMessenger) throw new Error("UIMessengerService is required");
    if (!profileService) throw new Error("ProfileService is required");
    var success = {}, error = {};
    var mergeRidesInto = function(rides, drive) {
        var wasModified = false;
        var srcRides = {};
        _.each(rides, function(ride) {
            srcRides[ride.rideId] = ride;
        });
        var driveRides = {};
        _.each(drive.rides, function(ride) {
            driveRides[ride.rideId] = ride;
        });
        _.each(drive.rides, function(ride) {
            if (!srcRides[ride.rideId]) {
                wasModified = true;
                boarding.updateRideState(ride.id, {
                    done: true,
                    error: true,
                    errorMessage: "Dropped off"
                });
            }
        });
        _.each(rides, function(ride) {
            if (!driveRides[ride.rideId]) {
                wasModified = true;
                var _drive = boarding.joinCar({
                    driveId: ride.driveId,
                    driverId: currentUser.id,
                    riderId: ride.riderId,
                    rideId: ride.rideId,
                    accuracy: ride.startAccuracy,
                    lat: ride.startLat,
                    lon: ride.startLon,
                    timestamp: ride.startTimestamp
                });
                ride = _.filter(_drive.rides, function(r) {
                    return ride.rideId == r.rideId;
                }).shift();
            }
        });
        return wasModified;
    };
    var setDriveAsInvalid = function(drive) {
        boarding.updateDriveState(drive.id, {
            done: true,
            error: true,
            errorMessage: "OutOfSync",
            errorCode: ""
        });
        _.each(drive.rides, function(ride) {
            boarding.updateRideState(ride.id, {
                done: true,
                error: true,
                errorMessage: "OutOfSync"
            });
        });
        summary.saveSummary({
            driveId: drive.id
        });
    };
    var setDriveAsCompleted = function(drive) {
        boarding.updateDriveState(drive.id, {
            done: true
        });
        _.each(drive.rides, function(ride) {
            boarding.updateRideState(ride.id, {
                done: true
            });
        });
        summary.saveSummary({
            driveId: drive.id
        });
    };
    success[APICallType.StartTrip] = function(apiCall, response) {
        var drive = apiCall.data.drive;
        boarding.updateDriveGPS(drive.id, response);
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    error[APICallType.StartTrip] = function(apiCall, networkError) {
        var drive = apiCall.data.drive, errorCode = networkError.code, errorMessage = networkError.message;
        boarding.updateDriveState(drive.id, {
            done: true,
            error: true,
            errorMessage: errorMessage,
            errorCode: errorCode
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    success[APICallType.EndTrip] = function(apiCall, response) {
        var drive = apiCall.data.drive;
        boarding.updateDriveState(drive.id, {
            done: true
        });
        var s = summary.saveSummary({
            driveId: drive.id,
            state: response.state,
            distance: response.distance,
            startTimestamp: response.startTimestamp,
            startAddress: response.startAddress,
            endTimestamp: response.endTimestamp,
            endAddress: response.endAddress,
            earning: response.earning,
            achievements: response.achievements,
            rides: response.rides,
            rewards: response.rewards,
            rewardTotals: response.rewardTotals,
            unrealisticCause: response.unrealisticCause
        });
        updateProfileCreditBalance(currentUser, profileService, response.creditBalance);
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    error[APICallType.EndTrip] = function(apiCall, networkError) {
        var drive = apiCall.data.drive, errorCode = networkError.code, errorMessage = networkError.message;
        var isError = true, actionWasIgnored = false;
        switch (errorCode) {
          case "future_end_time":
          case "missing_pickup_details":
          case "invalid_location":
          case "missing_pickup_details":
          case "temporarily_unavailable":
            isError = true;
            break;

          case "drive_inactive":
            isError = false;
            break;

          default:
            isError = true;
        }
        if (actionWasIgnored) return false;
        boarding.updateDriveState(drive.id, {
            done: true,
            error: isError,
            errorMessage: isError ? errorMessage : "",
            errorCode: isError ? errorCode : ""
        });
        summary.saveSummary({
            driveId: drive.id
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    success[APICallType.JoinCar] = function(apiCall, response) {
        var drive = apiCall.data.drive, ride = apiCall.data.ride;
        boarding.updateDriveGPS(drive.id, {
            driveId: response.drive.driverId
        });
        boarding.updateRideGPS(ride.id, {
            rideId: response.ride.rideId,
            driverId: response.driver.userId,
            driverFirstName: response.driver.firstName,
            driverAlias: response.driver.alias,
            driverPhotoUrl: response.driver.photoUrl
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    error[APICallType.JoinCar] = function(apiCall, networkError) {
        var drive = apiCall.data.drive, errorCode = networkError.code, errorMessage = networkError.message;
        switch (errorCode) {
          case "already_picked_up":
            return false;

          default:
            boarding.updateDriveState(drive.id, {
                done: true,
                error: true,
                errorMessage: errorMessage,
                errorCode: errorCode
            });
            drive = boarding.getDrive(drive.id);
            logger.log(function() {
                return BoardingLogMessages.fullDriveStatusLogMessage(drive);
            });
            uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
                drive: drive
            });
        }
    };
    success[APICallType.JoinCarByPhoneNumber] = function(apiCall, response) {
        var drive = apiCall.data.drive, ride = apiCall.data.ride;
        if (response.drive.driverId > 0 || -2 == response.drive.driverId) {
            boarding.updateDriveGPS(drive.id, response.drive);
            var phoneNumber = 0;
            apiCall && apiCall.data && (phoneNumber = apiCall.data.ride.driverPhoneNumber);
            profileService.saveMinimalProfile({
                firstName: response.driver.firstName,
                alias: response.driver.alias,
                photoUrl: response.driver.photoUrl,
                userId: response.drive.driverId,
                phoneNumber: phoneNumber
            });
        }
        boarding.updateRideGPS(ride.id, {
            rideId: response.ride.rideId
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    error[APICallType.JoinCarByPhoneNumber] = function(apiCall, networkError) {
        var drive = apiCall.data.drive, errorCode = networkError.code, errorMessage = networkError.message;
        boarding.updateDriveState(drive.id, {
            done: true,
            error: true,
            errorMessage: errorMessage,
            errorCode: errorCode
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    success[APICallType.JoinCarByBeacon] = function(apiCall, response) {
        var drive = apiCall.data.drive, ride = apiCall.data.ride;
        response.drive.driverId > 0 && boarding.updateDriveGPS(drive.id, response.drive);
        boarding.updateRideGPS(ride.id, {
            rideId: response.ride.rideId
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    error[APICallType.JoinCarByBeacon] = function(apiCall, networkError) {
        var drive = apiCall.data.drive, errorCode = networkError.code, errorMessage = networkError.message;
        boarding.updateDriveState(drive.id, {
            done: true,
            error: true,
            errorMessage: errorMessage,
            errorCode: errorCode
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    success[APICallType.LeaveCar] = function(apiCall, response) {
        var drive = apiCall.data.drive, ride = apiCall.data.ride;
        if (currentUser.id != drive.driverId) {
            boarding.updateDriveState(drive.id, {
                done: true
            });
            summary.saveSummary({
                driveId: drive.id,
                distance: response.distance,
                startTimestamp: response.startTimestamp,
                startAddress: response.startAddress,
                endTimestamp: response.endTimestamp,
                endAddress: response.endAddress,
                earning: response.earning,
                rewards: response.rewards,
                rewardTotals: response.rewardTotals,
                unrealisticCause: response.unrealisticCause,
                rides: [ {
                    driveId: drive.id,
                    riderId: currentUser.id,
                    duration: response.startTimestamp - response.endTimestamp,
                    freeRide: response.freeRide
                } ]
            });
            updateProfileCreditBalance(currentUser, profileService, response.creditBalance);
        } else boarding.updateRideState(ride.id, {
            done: true
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    error[APICallType.LeaveCar] = function(apiCall, networkError) {
        var drive = apiCall.data.drive, errorCode = networkError.code, errorMessage = networkError.message;
        var isError = true;
        switch (errorCode) {
          case "already_dropped_off":
            isError = false;
            break;

          default:
            isError = true;
        }
        boarding.updateDriveState(drive.id, {
            done: true,
            error: isError,
            errorMessage: isError ? errorMessage : "",
            errorCode: isError ? errorCode : ""
        });
        summary.saveSummary({
            driveId: drive.id
        });
        drive = boarding.getDrive(drive.id);
        logger.log(function() {
            return BoardingLogMessages.fullDriveStatusLogMessage(drive);
        });
        uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
            drive: drive
        });
    };
    success[APICallType.ActiveDrive] = function(apiCall, response) {
        var drives = response.drives, rides = response.rides, currentDrive = boarding.getCurrentDrive();
        var serverState = {
            isDriver: false,
            drive: null,
            isRider: false,
            ride: null,
            isLocalStateOutOfSync: false
        };
        drives && drives.length || !rides || !rides.length || rides[0].driverId || (drives = [].push({
            driverId: currentUser.id,
            driveId: rides[0].driveId,
            startTimestamp: rides[0].startTimestamp,
            startLat: rides[0].startLat,
            startLon: rides[0].startLon,
            startAccuracy: rides[0].startAccuracy
        }));
        drives.length && (rides = _.filter(rides, function(ride) {
            return ride.driveId == drives[0].driveId;
        }));
        if (drives.length) {
            var isLocalStateOutOfSync = true;
            if (currentDrive && currentDrive.driveId == drives[0].driveId) isLocalStateOutOfSync = false; else if (currentDrive && currentDrive.driveId != drives[0].driveId) {
                if (!currentDrive.driveId && currentDrive.driverId == currentUser.id) {
                    currentDrive = boarding.updateDriveGPS(currentDrive.id, drives[0]);
                    isLocalStateOutOfSync = false;
                }
            } else if (!currentDrive) {
                var localDriveExists = !!_.filter(boarding.getDrives(), function(d) {
                    return d.driveId == drives[0].driveId;
                }).length;
                isLocalStateOutOfSync = !localDriveExists;
            }
            _.extend(serverState, {
                isDriver: true,
                drive: drives[0],
                isLocalStateOutOfSync: isLocalStateOutOfSync
            });
        } else if (rides.length) {
            var isLocalStateOutOfSync = true, isRider = true, serverRide = rides[0];
            if (currentDrive && currentDrive.driverId == serverRide.driverId) {
                currentDrive = boarding.updateDriveGPS(currentDrive.id, serverRide);
                boarding.updateRideGPS(currentDrive.rides[0].id, serverRide);
                isLocalStateOutOfSync = false;
            } else if (currentDrive && currentDrive.driveId == serverRide.driveId) {
                isLocalStateOutOfSync = true;
                isRider = false;
                serverRide = null;
            }
            _.extend(serverState, {
                isRider: isRider,
                ride: serverRide,
                isLocalStateOutOfSync: isLocalStateOutOfSync
            });
        } else !currentDrive || drives.length || rides.length || _.extend(serverState, {
            isLocalStateOutOfSync: true
        });
        logger.log(_.template("APIUrls.<%= type %> Server State", {
            type: apiCall.type
        }));
        logger.log(function() {
            return BoardingLogMessages.driveStatusLogMessage(currentDrive);
        });
        logger.log(function() {
            return JSON.stringify(serverState, 0, 4);
        });
        !serverState.isLocalStateOutOfSync || serverState.isDriver || serverState.isRider ? serverState.isLocalStateOutOfSync && _.each(boarding.getDrives(), setDriveAsInvalid) : _.each(boarding.getDrives(), function(drive) {
            drive.id == currentDrive.id ? setDriveAsCompleted(drive) : setDriveAsInvalid(drive);
        });
        if (serverState.isLocalStateOutOfSync && serverState.isDriver) {
            var drive = boarding.startDrive({
                driverId: currentUser.id,
                timestamp: serverState.drive.startTimestamp
            });
            _.each(rides, function(ride) {
                boarding.joinCar({
                    driverId: currentUser.id,
                    driveId: ride.driveId,
                    rideId: ride.rideId,
                    riderId: ride.riderId,
                    accuracy: ride.startAccuracy,
                    lat: ride.startLat,
                    lon: ride.startLon,
                    timestamp: ride.startTimestamp
                });
            });
            boarding.updateDriveGPS(drive.id, serverState.drive);
        } else if (serverState.isLocalStateOutOfSync && serverState.isRider) {
            var drive = boarding.joinCar({
                rideId: serverState.ride.rideId,
                riderId: currentUser.id,
                driveId: serverState.ride.driveId,
                driverId: serverState.ride.driverId,
                accuracy: serverState.ride.startAccuracy,
                lat: serverState.ride.startLat,
                lon: serverState.ride.startLon,
                timestamp: serverState.ride.startTimestamp,
                freeRide: serverState.ride.freeRide
            });
            boarding.updateDriveGPS(drive.id, {
                driveId: serverState.ride.driveId,
                driverId: serverState.ride.driverId
            });
        }
        if (currentDrive && !serverState.isLocalStateOutOfSync && serverState.isDriver) {
            logger.log(_.template("APIUrls.<%= type %> Merging rides", {
                type: apiCall.type
            }));
            mergeRidesInto(rides, boarding.getDrive(currentDrive.id));
        }
        var currentDriveBefore = currentDrive ? boarding.getDrive(currentDrive.id) : null, currentDriveAfter = boarding.getCurrentDrive();
        if (serverState.isLocalStateOutOfSync && currentDriveBefore) {
            currentDriveBefore = boarding.getDrive(currentDriveBefore.id);
            logger.log(_.template("APIUrls.<%= type %> Setting current drive", {
                type: apiCall.type
            }));
            logger.log(function() {
                return BoardingLogMessages.fullDriveStatusLogMessage(currentDriveBefore);
            });
            uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
                drive: currentDriveBefore
            });
        }
        if (currentDriveAfter) {
            logger.log(_.template("APIUrls.<%= type %> Updating current drive", {
                type: apiCall.type
            }));
            logger.log(function() {
                return BoardingLogMessages.fullDriveStatusLogMessage(currentDriveAfter);
            });
            uiMessenger.send(UIMessageTypes.DriveStatusUpdated, {
                drive: currentDriveAfter
            });
        } else logger.log(_.template("APIUrls.<%= type %> No changes made!", {
            type: apiCall.type
        }));
    };
    error[APICallType.ActiveDrive] = function(apiCall, networkError) {};
    return {
        success: success,
        error: error
    };
};

module.exports = BoardingAPIResponseStrategies;