"use strict";

var _ = require("/vendor/underscore");

var EventEmitter = require("/v6/infrastructure/EventEmitter/Extend"), ExploreServiceBatchApiFactory = require("./ExploreServiceBatchApiFactory"), ExploreServiceBatchApiHelper = require("./ExploreServiceBatchApiHelper");

var ExploreService = function() {
    var eventDispatcher = new EventEmitter();
    var _BOOKINGS_READY = "bookings_ready", _searchTargetType = null, _selectedTripEffectiveTime = null, _selectedTrip = null, _skipFirstNextTripUpdatedEvent = true, _drivesMatchesPage = 1, _ridesMatchesPage = 1, _totalRidesMatchesPages = Number.MAX_VALUE, _totalDrivesMatchesPages = Number.MAX_VALUE, _BEST_MATCHES_DEFAULT_PAGE_SIZE = 10, _bookings = null;
    var _fetchExploreInfo = function() {
        _beforeBatchApiCall();
        var apiCallRequests = [];
        apiCallRequests.push(_generateBestMatchesBatchApiCallObject({
            searchTargetType: "RIDE"
        }));
        apiCallRequests.push(_generateBestMatchesBatchApiCallObject({
            searchTargetType: "DRIVE"
        }));
        apiCallRequests.push(_generateNextTripBatchApiCallObject());
        apiCallRequests.push(_generateBookingsBatchApiCallObject());
        var apiCallOrder = [ ExploreServiceBatchApiHelper.BEST_MATCHES_RIDE, ExploreServiceBatchApiHelper.BEST_MATCHES_DRIVE, ExploreServiceBatchApiHelper.NEXT_TRIP, ExploreServiceBatchApiHelper.BOOKINGS ];
        return ExploreServiceBatchApiFactory.execute(apiCallRequests, apiCallOrder, _batchApiCallback).then(function() {
            var response = {
                bestMatchesResponse: ExploreServiceBatchApiHelper.getBestMatchesResponse(),
                nextTripResponse: ExploreServiceBatchApiHelper.getNextTripResponse(),
                bookingsResponse: ExploreServiceBatchApiHelper.getBookingsResponse()
            };
            return response;
        });
    };
    var _beforeBatchApiCall = function() {
        _drivesMatchesPage = 1;
        _ridesMatchesPage = 1;
    };
    var _generateBestMatchesBatchApiCallObject = function(options) {
        options = options || {};
        if (!options.searchTargetType) throw new Error("[ExploreService] - searchTargetType not provided");
        var urlTemplateParam = {
            sortBy: "BEST_MATCH",
            pageSize: options.pageSize || _BEST_MATCHES_DEFAULT_PAGE_SIZE,
            pageNum: 1,
            nextTripTimestamp: _getSelectedTripEffectiveTime(),
            includeTypeCounts: true
        };
        _.extend(urlTemplateParam, options);
        var extraUrlParams = {
            tripId: _selectedTrip ? _selectedTrip.tripId : -1
        };
        return ExploreServiceBatchApiFactory.create("GET", "USER_TRIP_BEST_MATCHES", urlTemplateParam, extraUrlParams);
    };
    var _generateNextTripBatchApiCallObject = function(options) {
        options = options || {};
        var extraUrlParams = {
            tripFields: options.fields || "LOCATIONS,LOCATION_ADDRESSES,LOCATION_ALIASES,USER_ROLE,EFFECTIVE_TIME,BOOKINGS",
            relativeTime: _getSelectedTripEffectiveTime()
        };
        var urlKey = "USER_NEXT_TRIP";
        var urlTemplateParam = {};
        if (_selectedTrip && !_selectedTrip.enabled) {
            urlKey = "USER_TRIP_V2";
            urlTemplateParam.tripId = _selectedTrip.tripId;
        }
        return ExploreServiceBatchApiFactory.create("GET", urlKey, urlTemplateParam, extraUrlParams);
    };
    var _generateBookingsBatchApiCallObject = function(options) {
        options = options || {};
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId");
        var urlTemplateParam = {
            userId: userId
        };
        var extraUrlParams = {
            states: "ACCEPTED",
            userFields: "PHOTO_URL,ALIAS,PHONE_NUMBER,IS_FREE_DRIVER"
        };
        options.findAll && (extraUrlParams.states += ",REQUESTED");
        return ExploreServiceBatchApiFactory.create("GET", "USER_RIDE_BOOKING", urlTemplateParam, extraUrlParams);
    };
    var _batchApiCallback = function() {
        var bestMatchesResponse = ExploreServiceBatchApiHelper.getBestMatchesResponse(), nextTripResponse = ExploreServiceBatchApiHelper.getNextTripResponse();
        Alloy.createModel("trip_matches_explore").save(bestMatchesResponse, {
            localOnly: true
        });
        Alloy.createModel("trips_nextn").save(nextTripResponse, {
            localOnly: true
        });
        _saveProfiles(bestMatchesResponse);
        var rideMatches = bestMatchesResponse.rideMatches, driveMatches = bestMatchesResponse.driveMatches;
        _totalRidesMatchesPages = rideMatches.paginator.totalPages;
        _totalDrivesMatchesPages = driveMatches.paginator.totalPages;
    };
    var _saveProfiles = function(data) {
        var rideMatches = data.rideMatches, driveMatches = data.driveMatches;
        TM.push(function() {
            var Globals = require("/v6/infrastructure/Globals"), ProfileService = Globals.IOC.get("ProfileService");
            rideMatches && rideMatches.matches && _.each(rideMatches.matches, function(match) {
                match.user && ProfileService.saveMinimalProfile(match.user);
            });
            driveMatches && driveMatches.matches && _.each(driveMatches.matches, function(match) {
                match.user && ProfileService.saveMinimalProfile(match.user);
            });
        }, process.backgroundDomain);
        return data;
    };
    var _incrementPage = function() {
        if ("DRIVE" === _searchTargetType) {
            ++_drivesMatchesPage;
            return;
        }
        ++_ridesMatchesPage;
    };
    var _getBestMatchesPage = function() {
        if ("DRIVE" === _searchTargetType) return _drivesMatchesPage;
        return _ridesMatchesPage;
    };
    var _createNextTripModelAndInvokeMethod = function(options) {
        options = options || {};
        options.tripFields = options.tripFields || "LOCATIONS,LOCATION_ADDRESSES,LOCATION_ALIASES,USER_ROLE,EFFECTIVE_TIME,BOOKINGS";
        options.relativeTime = _getSelectedTripEffectiveTime();
        if (_selectedTrip && !_selectedTrip.enabled) {
            options.tripId = _selectedTrip.tripId;
            return Alloy.createModel("tripV2").getTrip(options);
        }
        var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId");
        return Alloy.createModel("trips_nextn", {
            ownerUid: userId
        }).getNextTrip(options);
    };
    var _bestMatches = function(customAttrs, customOpt) {
        customAttrs = customAttrs || {};
        customOpt = customOpt || {};
        var nextTripTimestamp = customOpt.nextPage ? (_selectedTripEffectiveTime || Date.now()) - 9e5 : _getSelectedTripEffectiveTime;
        delete customOpt.nextPage;
        var defaultAttrs = {
            sortBy: "BEST_MATCH",
            pageSize: customAttrs.pageSize || _BEST_MATCHES_DEFAULT_PAGE_SIZE,
            pageNum: _getBestMatchesPage(),
            nextTripTimestamp: nextTripTimestamp,
            includeTypeCounts: true
        };
        var attrs = _.extend({}, defaultAttrs, customAttrs);
        return Alloy.createModel("trip_matches_explore", attrs).bestMatches(customOpt).then(function(data) {
            if (data && 1 === _getBestMatchesPage()) {
                var rideMatches = data.rideMatches || {}, driveMatches = data.driveMatches || {}, tripMatchedAgainst = rideMatches.tripMatchedAgainst || driveMatches.tripMatchedAgainst || {};
                var nextTrip = tripMatchedAgainst.baseTrip || {}, resourceOwner = require("services/resource_owner_service").singleton();
                resourceOwner.set("nextTrip", nextTrip);
                resourceOwner.save();
                _totalRidesMatchesPages = rideMatches.paginator && rideMatches.paginator.totalPages;
                _totalDrivesMatchesPages = driveMatches.paginator && driveMatches.paginator.totalPages;
            }
            return data;
        });
    };
    var _nextTrip = function(options) {
        _bookings = null;
        return _createNextTripModelAndInvokeMethod(options).then(function(data) {
            data = data || {};
            _bookings = data.bookings || {};
            eventDispatcher.emit(_BOOKINGS_READY);
            return data;
        });
    };
    var _getBookingsForCurrentTrip = function() {
        var Q = require("vendor/q");
        if (_bookings) return Q.fcall(function() {
            return {
                bookings: _bookings
            };
        });
        var defer = Q.defer();
        eventDispatcher.once(_BOOKINGS_READY, function() {
            defer.resolve({
                bookings: _bookings
            });
        });
        return defer.promise;
    };
    var _getBestDriversAndRidersMatches = function() {
        _beforeBatchApiCall();
        var apiCallRequests = [];
        apiCallRequests.push(_generateBestMatchesBatchApiCallObject({
            searchTargetType: "RIDE"
        }));
        apiCallRequests.push(_generateBestMatchesBatchApiCallObject({
            searchTargetType: "DRIVE"
        }));
        var apiCallOrder = [ ExploreServiceBatchApiHelper.BEST_MATCHES_RIDE, ExploreServiceBatchApiHelper.BEST_MATCHES_DRIVE ];
        return ExploreServiceBatchApiFactory.execute(apiCallRequests, apiCallOrder, _batchApiCallback).then(function() {
            return ExploreServiceBatchApiHelper.getBestMatchesResponse();
        });
    };
    var _getBestMatches = function(options) {
        options = options || {};
        var nextPage = options.nextPage || false, localOnly = options.localOnly || false;
        if (nextPage) _incrementPage(); else {
            _drivesMatchesPage = 1;
            _ridesMatchesPage = 1;
        }
        var customOpts = {
            localOnly: localOnly,
            remoteOnly: nextPage,
            nextPage: nextPage
        };
        localOnly && (customOpts.query = {});
        return _bestMatches({
            searchTargetType: self.getSearchType()
        }, customOpts);
    };
    var _getSearchType = function() {
        return _searchTargetType;
    };
    var _setSearchType = function(searchType) {
        searchType = searchType || "RIDE_OR_DRIVE";
        _searchTargetType = searchType;
        eventDispatcher.emit(self.SEARCH_TYPE_CHANGED);
    };
    var _resetSearchType = function() {
        _searchTargetType = null;
    };
    var _isThereNextBestMatchesPageForType = function(searchType) {
        if (!searchType) {
            LOGGER.error("[ExploreService] - No searchType provided");
            return false;
        }
        if ("DRIVE" === searchType) return _totalDrivesMatchesPages > _drivesMatchesPage;
        return _totalRidesMatchesPages > _ridesMatchesPage;
    };
    var _getSelectedTripEffectiveTime = function() {
        var nowMinus15 = Date.now() - 9e5;
        if (_selectedTripEffectiveTime) {
            if (nowMinus15 > _selectedTripEffectiveTime) {
                _selectedTripEffectiveTime = null;
                _selectedTrip = null;
                return nowMinus15;
            }
            return _selectedTripEffectiveTime;
        }
        return nowMinus15;
    };
    var _setSelectedTripEffectiveTime = function(trip) {
        if (!trip || !trip.schedule.effectiveTime) {
            LOGGER.error("[ExploreService.js] - trip effectiveTime is null or not provided");
            return false;
        }
        var currentSelectedTrip = _selectedTrip || {};
        if (JSON.stringify(currentSelectedTrip) === JSON.stringify(trip)) return false;
        _selectedTripEffectiveTime = trip.schedule.effectiveTime;
        _selectedTrip = trip;
        self.resetSearchType();
        _skipFirstNextTripUpdatedEvent || eventDispatcher.emit(self.NEXT_TRIP_UPDATED);
        _skipFirstNextTripUpdatedEvent = false;
        return true;
    };
    var _clearSelectedTripEffectiveTime = function() {
        _selectedTripEffectiveTime = null;
        _selectedTrip = null;
        _skipFirstNextTripUpdatedEvent || eventDispatcher.emit(self.NEXT_TRIP_UPDATED);
        _skipFirstNextTripUpdatedEvent = false;
    };
    var _getSelectedTrip = function() {
        return _selectedTrip;
    };
    var self = {
        FILTERS_UPDATED: "filters_updated",
        SEARCH_TYPE_CHANGED: "search_type_changed",
        NEXT_TRIP_UPDATED: "next_trip_updated",
        fetchExploreInfo: function() {
            return _fetchExploreInfo();
        },
        nextTrip: function(options) {
            return _nextTrip(options);
        },
        getBookingsForCurrentTrip: function() {
            return _getBookingsForCurrentTrip();
        },
        getBestDriversAndRidersMatches: function() {
            return _getBestDriversAndRidersMatches();
        },
        getBestMatches: function(options) {
            return _getBestMatches(options);
        },
        getSearchType: function() {
            return _getSearchType();
        },
        setSearchType: function(searchType) {
            _setSearchType(searchType);
        },
        resetSearchType: function() {
            _resetSearchType();
        },
        isThereNextBestMatchesPageForType: function(searchType) {
            return _isThereNextBestMatchesPageForType(searchType);
        },
        on: function(event, callback) {
            eventDispatcher.on(event, callback);
        },
        setSelectedTripEffectiveTime: function(trip) {
            return _setSelectedTripEffectiveTime(trip);
        },
        clearSelectedTripEffectiveTime: function() {
            _clearSelectedTripEffectiveTime();
        },
        getSelectedTrip: function() {
            return _getSelectedTrip();
        }
    };
    return self;
};

ExploreService.$isRevealingModule = true;

module.exports = ExploreService;