"use strict";

var _ = require("/vendor/underscore"), EventEmitter = require("/v6/infrastructure/EventEmitter/Extend"), Globals = require("/v6/infrastructure/Globals"), completenessService = Globals.IOC.get("completenessService");

var MessageService = function() {};

MessageService.prototype = new EventEmitter();

MessageService.prototype.constructor = MessageService;

var chatModel = null, conversationCollection = null;

_.extend(MessageService.prototype, {
    MESSAGES_UPDATED_EVENT: "messages_updated",
    unreadCount: function() {
        var resourceOwner = require("services/resource_owner_service").singleton();
        return Alloy.createModel("user_messages", {
            urlParams: {
                userId: resourceOwner.get("userId")
            }
        }).countUnread();
    },
    markThreadRead: function(options) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        return Alloy.createModel("user_messages").markThreadRead({
            userId: resourceOwner.get("userId"),
            threadId: options.threadId
        });
    },
    countThreadUnread: function(options) {
        var resourceOwner = require("services/resource_owner_service").singleton();
        return Alloy.createModel("user_messages").countThreadUnread({
            userId: resourceOwner.get("userId"),
            threadId: options.threadId
        });
    },
    send: function(userId, message) {
        var resourceOwner = require("services/resource_owner_service").singleton(), senderId = resourceOwner.get("userId"), self = this;
        return Alloy.createModel("user_messages", {
            urlParams: {
                userId: userId
            },
            messageType: "USER_TEXT",
            sendingUser: senderId,
            body: message,
            recipientUser: userId
        }).save().then(function(data) {
            self._insertMessageInCache(data);
            completenessService.refresh();
            return data;
        });
    },
    sendGroup: function(groupId, message) {
        var resourceOwner = require("services/resource_owner_service").singleton(), senderId = resourceOwner.get("userId"), self = this;
        return Alloy.createModel("user_messages").sendGroupMessage({
            groupId: groupId,
            messageType: "USER_TEXT",
            sendingUser: senderId,
            body: message
        }).then(function(data) {
            self._insertMessageInCache(data, groupId);
            completenessService.refresh();
            return data;
        });
    },
    deleteChat: function(options) {
        var self = this;
        options = options || {};
        options.userId = require("services/resource_owner_service").singleton().get("userId");
        return Alloy.createModel("user_messages", {
            id: options.threadId
        }).deleteChat(options).then(function(data) {
            self.emitEvent(self.MESSAGES_UPDATED_EVENT, true);
            return data;
        });
    },
    poke: function(attr) {
        var self = this;
        attr = attr || {};
        attr.sendingUser = require("services/resource_owner_service").singleton().get("userId");
        attr.timestamp = Date.now();
        attr.messageType = "USER_POKE";
        return Alloy.createModel("user_messages").poke(attr).then(function(data) {
            self._insertMessageInCache(data);
            completenessService.refresh();
            return data;
        });
    },
    getConversations: function() {
        conversationCollection = Alloy.createCollection("user_conversations");
        return this.getCached().then(function(cache) {
            var timestampsCache = [];
            cache && cache.length > 0 && cache.forEach(function(item) {
                timestampsCache.push(item.messages.notifications[0].timestamp);
            });
            return conversationCollection.fetch().then(function(data) {
                var timestamps = [];
                data && data.length > 0 && data.forEach(function(item) {
                    timestamps.push(item.messages.notifications[0].timestamp);
                });
                if (timestampsCache.length && timestampsCache.toString() === timestamps.toString()) return -1;
                return data;
            });
        });
    },
    getCached: function() {
        conversationCollection = Alloy.createCollection("user_conversations");
        return conversationCollection.fetch({
            localOnly: true
        });
    },
    getPageSize: function() {
        return Alloy.createModel("user_chat")._pageSize;
    },
    getNextConversationPage: function() {
        return conversationCollection.next();
    },
    conversationHasNextPage: function() {
        return conversationCollection.hasNext();
    },
    getChatMessages: function(options) {
        chatModel = Alloy.createModel("user_chat", {
            id: options.threadId
        });
        return chatModel.fetch(options);
    },
    getNextChatMessagesPage: function(page) {
        return chatModel.next();
    },
    chatHasNextPage: function() {
        return chatModel.hasNext();
    },
    getNewChatMessages: function(options) {
        return chatModel.fetch(options);
    },
    updateConversationsCache: function(threadId) {
        return Alloy.createCollection("user_conversations").fetch();
    },
    destroy: function() {
        this.removeEvent();
    },
    _insertMessageInCache: function(data, groupId) {
        var self = this;
        this.getCached().then(function(conversations) {
            var threadId = groupId || (data.recipientUser > data.sendingUser ? data.recipientUser + "_" + data.sendingUser : data.sendingUser + "_" + data.recipientUser);
            var conv = _.findWhere(conversations, {
                id: threadId
            });
            if (conv) {
                _.extend(conv.messages.notifications[0], {
                    id: data.response,
                    body: data.body,
                    messageType: data.messageType,
                    recipientUser: data.recipientUser || data.sendingUser,
                    sendingUser: data.sendingUser,
                    timestamp: Date.now()
                });
                Alloy.createModel("user_conversations").save(conv, {
                    localOnly: true
                }).then(function() {
                    self.emitEvent(self.MESSAGES_UPDATED_EVENT, [ {
                        localOnly: true
                    } ], true);
                });
            }
        });
    }
});

module.exports = MessageService;