"use strict";

var Q = require("/vendor/q"), _ = require("/vendor/underscore"), APICall = require("./entity/APICall"), APICallType = require("./entity/APICallType"), APIUrls = require("./entity/APIUrls"), NetworkError = require("/v6/common/errors/NetworkError"), TimeManager = require("./helpers/TimeManager");

var DEFAULT_TIME_OUT = 6e4;

var HTTP_HEADERS = {
    "Content-Type": "application/json;charset=UTF-8",
    Accept: "application/json",
    "User-Agent": Ti.userAgent,
    "Accept-Encoding": "gzip,deflate"
};

var getResponse = function(response, formatter) {
    var responseText = response.source.responseText;
    if (!responseText) throw new Error(response.error);
    var json = JSON.parse(responseText);
    return formatter(json);
};

var processBatchResponse = function(response, apiCalls, apiUrlArgs) {
    var responseText = response.source.responseText;
    if (!responseText) throw new Error(response.error);
    var responseData = [];
    var json = JSON.parse(response.source.responseText).apiCallResponses;
    for (var i = 0; i < apiCalls.length; i++) {
        var call = apiCalls[i];
        call = APIUrls[call.type];
        var dataToProcess = JSON.parse(json[i].body);
        var respData = call.getResponseData(dataToProcess, apiUrlArgs);
        responseData.push(respData);
    }
    return responseData;
};

var createNetworkError = function(data, innerError) {
    data = _.extend({}, data, {
        url: data.url,
        verb: data.verb
    });
    return new NetworkError(data, innerError);
};

var NetworkService = function(currentUser, config, loggingService) {
    if (null == currentUser) throw new Error("UserService is required!");
    if (null == config) throw new Error("ConfigService is required!");
    if (null == loggingService) throw new Error("LoggingService is required!");
    var logger = loggingService.logger("network"), timeManager = new TimeManager();
    var getUpdateServerTimeApiCall = function() {
        if (!timeManager.isServerTimeValid()) return new APICall(APICallType.SystemDetails, Date.now(), {});
        return null;
    };
    var generateBatchCallData = function(apiCalls) {
        var apiCallRequests = [];
        _.each(apiCalls, function(call) {
            var apiUrl = APIUrls[call.type];
            var apiUrlArgs = {
                bindArgs: [ networkService ]
            };
            var data = _.extend({}, call.data, {
                userId: currentUser.id
            });
            var url = config.baseUrl + apiUrl.getUrl(data, apiUrlArgs);
            apiCallRequests.push({
                method: apiUrl.verb(),
                url: url
            });
        });
        var req = {
            apiCallRequests: apiCallRequests
        };
        return JSON.stringify(req);
    };
    var makeRequest = function(apiCalls, options) {
        options = options || {};
        var dfd = Q.defer();
        var isBatchCall = apiCalls.length > 1 ? true : false;
        var latencyRefDate = null, apiUrl = APIUrls[apiCalls[0].type], apiUrlArgs = {
            bindArgs: [ networkService ]
        }, data = _.extend({}, apiCalls[0].data, {
            userId: currentUser.id
        });
        if (isBatchCall) {
            apiUrl = APIUrls[APICallType.BatchAPICall];
            data = _.extend({}, data, {
                userId: currentUser.id
            });
        }
        var url = config.baseUrl + apiUrl.getUrl(data, apiUrlArgs), verb = apiUrl.verb();
        var client = Ti.Network.createHTTPClient({
            cache: options.cache || false,
            enableKeepAlive: options.enableKeepAlive || true,
            timeout: options.timeout || DEFAULT_TIME_OUT
        });
        var sharedHandler = function(fn) {
            return function(response) {
                var latencyTime = Date.now() - latencyRefDate;
                logger.log(_.template("	 <= <%= url %> [TIcode: <%= code %>] [duration: <%= duration %>]", {
                    url: url,
                    duration: latencyTime,
                    code: response.code
                }));
                var serverDateStr = null;
                try {
                    serverDateStr = response.source.getResponseHeader("Date");
                } catch (e) {}
                if (serverDateStr) {
                    var serverDate = new Date(serverDateStr).getTime();
                    if (1e4 > latencyTime) {
                        serverDate += latencyTime / 2;
                        timeManager.setServerTime(serverDate);
                    }
                }
                try {
                    fn(response);
                } catch (e) {
                    var isOffline = 1 == response.code || 2 == response.code || false === response.source.connected;
                    var error = createNetworkError({
                        url: url,
                        verb: verb,
                        isOffline: isOffline
                    }, e);
                    dfd.reject(error);
                }
            };
        };
        client.onload = sharedHandler(function(response) {
            try {
                if (isBatchCall) {
                    var responseData = processBatchResponse(response, apiCalls, apiUrlArgs);
                    dfd.resolve(responseData);
                } else {
                    var data = getResponse(response, function(data) {
                        return apiUrl.getResponseData(data, apiUrlArgs);
                    });
                    dfd.resolve(data);
                }
            } catch (e) {
                var error = createNetworkError({
                    url: url,
                    verb: verb
                }, e);
                dfd.reject(error);
            }
        });
        client.onerror = sharedHandler(function(response) {
            var data = getResponse(response, function(data) {
                return apiUrl.getErrorData(data, apiUrlArgs);
            });
            var error = createNetworkError(_.extend(data, {
                url: url,
                verb: verb
            }));
            dfd.reject(error);
        });
        client.open(verb, url);
        var headers = _.extend({}, options.headers, HTTP_HEADERS);
        currentUser.token && (headers.Authorization = "Bearer " + currentUser.token);
        _.each(headers, function(value, header) {
            client.setRequestHeader(header, value);
        });
        latencyRefDate = Date.now();
        logger.log(url);
        "GET" == verb ? client.send() : client.send(isBatchCall ? generateBatchCallData(apiCalls) : apiUrl.getPostData(data, apiUrlArgs));
        return dfd.promise;
    };
    var networkService = {
        needToSyncTime: function() {
            return !timeManager.isServerTimeValid();
        },
        getOffset: function() {
            return timeManager.getOffset();
        },
        getTime: function(date) {
            return timeManager.getTime(date);
        },
        request: function(apiCall, options) {
            var calls = [];
            apiCall instanceof Array ? calls = apiCall : apiCall && calls.push(apiCall);
            return Q().then(function() {
                var sysApiCall = getUpdateServerTimeApiCall();
                if (sysApiCall) return makeRequest(new Array(sysApiCall));
            }).catch(function() {}).then(function() {
                if (calls.length) return makeRequest(calls, options);
            });
        }
    };
    return networkService;
};

NetworkService.$inject = [ "CurrentUserService", "ConfigService", "LoggingService" ];

NetworkService.$isRevealingModule = true;

module.exports = NetworkService;