"use strict";

var _ = require("/vendor/underscore"), Q = require("/vendor/q"), Profile = require("./models/Profile"), Trip = require("./models/Trip"), Card = require("./models/Card"), Vehicle = require("./models/Vehicle"), CreditBalance = require("./models/CreditBalance"), MessagingStats = require("./models/MessagingStats"), Validations = require("./models/Validations"), Schedule = require("./models/Schedule"), TripCollection = require("./models/TripCollection"), CardCollection = require("./models/CardCollection"), VehicleCollection = require("./models/VehicleCollection"), ProfileCollection = require("./models/ProfileCollection");

var APICallResponseStrategy = require("/v6/network/strategy/APICallResponseStrategy"), ProfileApiCallFactory = require("/v6/profile/helpers/ProfileApiCallFactory"), ProfileApiResponseStrategies = require("/v6/profile/strategy/ProfileApiResponseStrategies");

var InvalidArguments = require("../common/errors/InvalidArguments");

var Internal = {};

var ProfileService = function(currentUserService, network, loggingService) {
    if (!currentUserService) throw new Error("CurrentUserService is required");
    if (!network) throw new Error("NetworkService is required");
    if (!loggingService) throw new Error("LoggingService is required");
    var logger = loggingService.logger("profile");
    var strategy = new APICallResponseStrategy(new ProfileApiResponseStrategies(), logger);
    var TRIP_DETAIL = "FULL_TRIPS";
    var FULL_DETAIL = "FULL";
    var BASIC_DETAIL = "BASIC";
    var findProfile = function(phoneNumber) {
        var profiles = new ProfileCollection().fetch();
        var models = profiles.models;
        var foundModels = profiles.filter(function(data) {
            return phoneNumber.test(data.get("phoneNumber"));
        });
        if (foundModels.length > 0) return foundModels[0];
        return null;
    };
    var doesProfileNeedRefresh = function(profile, level) {
        if (null === profile) return true;
        var now = Date.now();
        if (now > profile.get("expiry")) return true;
        if (level === FULL_DETAIL) {
            if (!profile.get("phoneNumber") && !profile.get("messagingStats")) return true;
        } else if (level === TRIP_DETAIL && !profile.get("trips")) return true;
        return false;
    };
    var renewExpiry = function() {
        var expiry = 288e5;
        expiry = Date.now() + expiry;
        return expiry;
    };
    var transformProfileObject = function(profile) {
        profile.id = profile.userId;
        var location = profile.lastKnownLocation;
        if (location) {
            profile.lastKnownLocation = location.address;
            profile.lastSeen = location.timestamp;
        }
        var homeCity = profile.homeCity;
        if (homeCity) {
            profile.homeCity = homeCity.name;
            profile.homeCountry = homeCity.country.name;
        }
        var validations = profile.validations;
        if (validations) {
            profile.validations = {};
            profile.validations.phoneValidated = validations.phoneValidated;
            profile.validations.emailValidated = validations.emailValidated;
            profile.validations.numPositiveReviews = validations.numPositiveReviews;
            profile.validations.connectedFacebook = validations.connectedFacebook;
            profile.validations.numberOfFaceBookFriends = validations.numberOfFaceBookFriends;
            profile.affiliateNames = validations.affiliateNames;
        }
        if (profile.creditBalances && profile.creditBalances.length > 0) {
            profile.creditBalance = {};
            profile.creditBalance.currency = profile.creditBalances[0].currency;
            profile.creditBalance.currencySymbol = profile.creditBalances[0].currencySymbol;
            profile.creditBalance.bonusBalance = profile.creditBalances[0].bonusBalance;
            profile.creditBalance.payableBalance = profile.creditBalances[0].payableBalance;
            profile.creditBalance.totalBalance = profile.creditBalances[0].totalBalance;
        }
        profile.earnedAchievements = null;
        profile.achievements = null;
        profile.countryDetails = null;
        return profile;
    };
    var saveMinimalProfile = function(options) {
        if (options.lastKnownLocation) {
            options.lastSeen = options.lastKnownLocation.timestamp;
            options.lastKnownLocation = options.lastKnownLocation.address;
        }
        var profile = new Profile(options);
        profile.save();
        return profile;
    };
    var getCachedProfile = function(id) {
        var profile = new Profile().fetch({
            id: id
        });
        if (profile) return profile;
        if (id === currentUserService.id) {
            var resourceOwner = require("/services/resource_owner_service").singleton();
            if (id === resourceOwner.get("userId")) {
                profile = saveProfile([ transformProfileObject(resourceOwner.toJSON()) ], FULL_DETAIL, {
                    force: true
                });
                return profile;
            }
        }
        return null;
    };
    var saveProfile = function(data, level, params) {
        params = params || {};
        var profileData = data[0];
        var tripData = null;
        var cardData = null;
        data.length > 1 && (level === TRIP_DETAIL ? tripData = data[1] : cardData = data[1]);
        var profile = null;
        params.force || getCachedProfile(profileData.id);
        var expiry = renewExpiry();
        var options = _.extend({}, {
            userId: profileData.id,
            alias: profileData.alias || null,
            phoneNumber: profileData.phoneNumber || null,
            firstName: profileData.firstName || null,
            lastName: profileData.lastName || null,
            webpage: profileData.webpage || null,
            bio: profileData.bio || null,
            registrationTime: profileData.registrationTime || 0,
            homeCity: profileData.homeCity || null,
            homeCountry: profileData.homeCountry || null,
            lastKnownLocation: profileData.lastKnownLocation || null,
            lastSeen: profileData.lastSeen || 0,
            gender: profileData.gender || null,
            photoUrl: profileData.photoUrl || null,
            isFavourite: profileData.isFavourite || false,
            numFavourites: profileData.numFavourites || 0,
            canUserTakeFreeRide: profileData.canUserTakeFreeRide || false,
            canUserGiveFreeDrive: profileData.canUserGiveFreeDrive || false,
            expiry: expiry,
            numSharedRides: profileData.numSharedRides || 0,
            numSharedDrives: profileData.numSharedDrives || 0,
            totalRides: profileData.numSharedRides + profileData.numSharedDrives,
            credit: profileData.credit || 0,
            avgUserRating: profileData.avgUserRating || 0,
            numUserRatings: profileData.numUserRatings || 0,
            numUserComments: profileData.numUserComments || 0,
            affiliateNames: profileData.affiliateNames || [],
            beacons: profileData.beacons || []
        });
        options.groups = profileData.groups ? profileData.groups.length : 0;
        null === profile && (profile = new Profile());
        profile.set(options);
        if (profileData.creditBalance) {
            var creditBalance = new CreditBalance(profileData.creditBalance).save();
            profile.setCreditBalance(creditBalance);
        }
        if (profileData.messagingStats) {
            var messagingStats = new MessagingStats(profileData.messagingStats).save();
            profile.setMessagingStats(messagingStats);
        }
        if (profileData.validations) {
            var validations = new Validations(profileData.validations).save();
            profile.setValidations(validations);
        }
        if (profileData.vehicles) {
            var vehicles = new VehicleCollection();
            for (var i = 0; i < profileData.vehicles.length; i++) vehicles.push(new Vehicle(profileData.vehicles[i]).save());
            profile.setVehicles(vehicles);
        }
        if (tripData) {
            var tripCollection = new TripCollection();
            for (var t = 0; t < tripData.trips.length; t++) tripData.trips[t].enabled && tripCollection.push(createTripObject(tripData.trips[t]));
            profile.setTrips(tripCollection);
        }
        if (cardData) {
            var cardCollection = new CardCollection();
            for (var y = 0; y < cardData.tokens.length; y++) cardCollection.push(createCardObject(cardData.tokens[y]));
            profile.setCards(cardCollection);
        }
        profile.save();
        return profile;
    };
    var createTripObject = function(data) {
        var trip = new Trip();
        var options = {
            tripId: data.id || -1,
            userRole: data.userRole,
            enabled: data.enabled || false
        };
        trip.set(options);
        var scheduleOptions = {
            type: data.schedule.type || null,
            daysOfOperation: data.schedule.daysOfOperation || [],
            startMinutes: data.schedule.startMinutes || 0,
            timeZone: data.schedule.timeZone || null
        };
        var schedule = new Schedule(scheduleOptions).save();
        trip.setSchedule(schedule);
        var earning = new CreditBalance(data.estimatedEarning).save();
        trip.setEstimatedEarning(earning);
        var cost = new CreditBalance(data.estimatedCost).save();
        trip.setEstimatedEarning(cost);
        return trip.save();
    };
    var createCardObject = function(data) {
        var card = new Card();
        var options = {
            id: data.id || -1,
            brand: data.brand,
            alias: data.alias || false,
            metadata: data.metadata || null,
            paymentProvider: data.paymentProvider || null,
            providerAccountCurrencyCode: data.providerAccountCurrencyCode || null,
            status: data.status || null,
            timestamp: data.timestamp || -1,
            token: data.token || null,
            userId: data.userId || -1
        };
        card.set(options);
        return card.save();
    };
    var updateCredit = function(profile, bonus, payable, total) {
        if (!total) {
            total = profile.get("credit");
            total += bonus;
        }
        profile.set("credit", total);
        var creditBalance = profile.get("creditBalance");
        if (creditBalance) {
            payable && creditBalance.set("payableBalance", payable);
            creditBalance.set("bonusBalance", bonus);
            creditBalance.set("totalBalance", total);
            creditBalance.save();
        }
        profile.set("expiry", renewExpiry());
        profile.save();
    };
    var updateCards = function(profile, cards) {
        var cardCollection = new CardCollection();
        for (var t = 0; t < cards.length; t++) cardCollection.push(createCardObject(cards[t]));
        profile.setCards(cardCollection);
        profile.set("expiry", renewExpiry());
        profile.save();
    };
    var self = {
        FULL_DETAIL: FULL_DETAIL,
        BASIC_DETAIL: BASIC_DETAIL,
        TRIP_DETAIL: TRIP_DETAIL,
        saveProfileToCache: function(profile) {
            var cached = getCachedProfile(profile.userId);
            (!cached || doesProfileNeedRefresh(cached)) && saveProfile([ transformProfileObject(profile) ], FULL_DETAIL);
        },
        saveMinimalProfile: function(options) {
            var cached = getCachedProfile(options.userId);
            cached || saveMinimalProfile(options);
        },
        toggleFavourite: function(ownerProfileId, targetProfileId, markFavourite) {
            var ownerProfile = getCachedProfile(ownerProfileId);
            if (ownerProfile) {
                var numFavourites = ownerProfile.get("numFavourites");
                numFavourites = markFavourite ? numFavourites + 1 : numFavourites - 1;
                ownerProfile.set("numFavourites", numFavourites);
                ownerProfile.set("expiry", renewExpiry());
                ownerProfile.save();
            }
            var targetProfile = getCachedProfile(targetProfileId);
            if (targetProfile) {
                targetProfile.set("isFavourite", markFavourite);
                targetProfile.set("expiry", renewExpiry());
                targetProfile.save();
            }
        },
        updateFreeRide: function(targetProfileId, canFreeRide) {
            var userProfile = getCachedProfile(targetProfileId);
            if (userProfile) {
                userProfile.set("canUserTakeFreeRide", canFreeRide);
                userProfile.set("expiry", renewExpiry());
                userProfile.save();
            }
        },
        updateCredit: function(userId, total, payable, bonus) {
            var cached = getCachedProfile(userId);
            cached && updateCredit(cached, bonus, payable, total);
        },
        updateCards: function(userId, cards) {
            var cached = getCachedProfile(userId);
            cached && updateCards(cached, cards);
        },
        updateBonusBalance: function(userId, bonus) {
            var cached = getCachedProfile(userId);
            cached && updateCredit(cached, bonus);
        },
        getCachedProfile: function(userId) {
            var cached = getCachedProfile(userId);
            if (cached) return cached.toJSON();
            return null;
        },
        getProfileByPhoneNumber: function(phoneNumber) {
            phoneNumber && phoneNumber.length > 1 && (phoneNumber = phoneNumber.substring(1));
            var phoneNumberRegex = new RegExp(".*" + phoneNumber);
            var profile = findProfile(phoneNumberRegex);
            if (profile) return profile.toJSON();
            return null;
        },
        getUserProfiles: function(userIds) {
            var profileList = [];
            var apiFactory = new ProfileApiCallFactory();
            var dfd = Q.defer();
            if (!userIds || 0 == userIds.length) {
                dfd.reject(null);
                return dfd.promise;
            }
            var cachedIds = [];
            for (var i = 0; i < userIds.length; i++) {
                var cached = getCachedProfile(userIds[i]);
                if (cached) {
                    profileList.push(cached.toJSON());
                    cachedIds.push(userIds[i]);
                }
            }
            var uidsToCheck = _.difference(userIds, cachedIds);
            if (uidsToCheck.length > 0) {
                var uids = uidsToCheck.join(",");
                var apiCall = apiFactory.createProfileListCall(uids);
                network.request(apiCall).then(function(data) {
                    try {
                        for (var i = 0; i < data.users.length; i++) {
                            var p = saveMinimalProfile(data.users[i]);
                            profileList.push(p.toJSON());
                        }
                        dfd.resolve(profileList);
                    } catch (e) {
                        dfd.reject(e);
                    }
                }).catch(function(error) {
                    error.isOffline && dfd.reject(error);
                });
            } else dfd.resolve(profileList);
            return dfd.promise;
        },
        getProfile: function(userId, level, forceFromServer) {
            var apiFactory = new ProfileApiCallFactory();
            var dfd = Q.defer();
            if (!userId || 0 >= userId) {
                dfd.reject(null);
                return dfd.promise;
            }
            var cachedVersion = getCachedProfile(userId);
            if (doesProfileNeedRefresh(cachedVersion, level) || forceFromServer) {
                var callSequence = [];
                if (level === TRIP_DETAIL) {
                    callSequence.push(apiFactory.createFullProfileCall(userId));
                    callSequence.push(apiFactory.createGetTripsCall(userId));
                } else if (level === FULL_DETAIL) {
                    callSequence.push(apiFactory.createFullProfileCall(userId));
                    callSequence.push(apiFactory.createGetCardsCall(userId));
                } else callSequence.push(apiFactory.createBasicProfileCall(userId));
                network.request(callSequence).then(function(data) {
                    try {
                        var responses = [];
                        if (data instanceof Array) for (var i = 0; i < callSequence.length; i++) responses.push(strategy.execute(callSequence[i], data[i])); else responses.push(strategy.execute(callSequence[0], data));
                        var profile = saveProfile(responses, level);
                        dfd.resolve(profile.toJSON());
                    } catch (e) {
                        dfd.reject(e);
                    }
                }).catch(function(error) {
                    error.isOffline && (cachedVersion ? dfd.resolve(cachedVersion.toJSON()) : dfd.reject(error));
                });
            } else dfd.resolve(cachedVersion.toJSON());
            return dfd.promise;
        }
    };
    if ("function" == typeof logger.logDuration) {
        _.each(self, function(fn, name) {
            self[name] = logger.logDuration(fn, {
                name: name,
                scope: self
            });
        });
        saveProfile = logger.logDuration(saveProfile, {
            name: "saveProfile"
        });
        createCardObject = logger.logDuration(createCardObject, {
            name: "createCardObject"
        });
        createTripObject = logger.logDuration(createTripObject, {
            name: "createTripObject"
        });
        updateCredit = logger.logDuration(updateCredit, {
            name: "updateCredit"
        });
        updateCards = logger.logDuration(updateCards, {
            name: "updateCards"
        });
    }
    return self;
};

ProfileService.$inject = [ "CurrentUserService", "NetworkService", "LoggingService" ];

ProfileService.$isRevealingModule = true;

module.exports = ProfileService;