"use strict";

var EventEmitter = require("/v6/infrastructure/EventEmitter/Extend"), _ = require("/vendor/underscore"), Q = require("/vendor/q");

var BookingService = function() {
    var eventDispatcher = new EventEmitter();
    var _compareBooking = function(booking1, booking2) {
        if (!booking1 || !booking2) return;
        var priorities = {
            REQUESTED: 2,
            ACCEPTED: 1
        };
        var priorityBooking1 = priorities[booking1.state] || 0, priorityBooking2 = priorities[booking2.state] || 0;
        if (priorityBooking1 > priorityBooking2) return booking1;
        if (priorityBooking2 > priorityBooking1) return booking2;
        return booking1.timestamp > booking2.timestamp ? booking1 : booking2;
    };
    return {
        BOOKING_UPDATED: "booking_updated",
        bookingsRecord: function() {
            var rideBookingRecord = Alloy.createModel("ride_booking_record");
            rideBookingRecord.fetch();
            return rideBookingRecord;
        },
        findAll: function(onlyConfirmed) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            var userId = resourceOwner.get("userId");
            return Alloy.createModel("ride_booking").findAll(userId, onlyConfirmed);
        },
        findAllStates: function(tripId, baseEffectiveTime) {
            var userId = require("services/resource_owner_service").singleton().get("userId"), states = "REQUESTED,ACCEPTED,COMPLETED_RIDER_PICKUP,COMPLETED_DRIVER_PICKUP,DECLINED,CANCELLED,CANCELLED_TOO_OLD";
            return Alloy.createModel("ride_booking").findByStates(userId, tripId, baseEffectiveTime, states, true);
        },
        findByTrip: function(tripId, onlyConfirmed) {
            var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), statesQuery = "ACCEPTED";
            onlyConfirmed || (statesQuery += ",REQUESTED");
            return Alloy.createModel("trip_ride_booking", {
                userId: userId,
                tripId: tripId,
                statesQuery: statesQuery
            }).fetch();
        },
        rideRequest: function(trip, groupingKey) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            var userId = resourceOwner.get("userId");
            var riderId, driverId;
            riderId = userId;
            driverId = trip.user.userId;
            return Alloy.createModel("ride_booking", {
                urlParams: {
                    userId: userId
                },
                tripId: trip.id,
                riderId: riderId,
                driverId: driverId,
                bookingDate: trip.effectiveTime,
                groupingKey: groupingKey
            }).save();
        },
        rideOffer: function(trip, groupingKey) {
            var resourceOwner = require("services/resource_owner_service").singleton();
            var userId = resourceOwner.get("userId");
            var riderId, driverId;
            riderId = trip.user.userId;
            driverId = userId;
            return Alloy.createModel("ride_booking", {
                urlParams: {
                    userId: userId
                },
                tripId: trip.id,
                riderId: riderId,
                driverId: driverId,
                bookingDate: trip.effectiveTime,
                groupingKey: groupingKey
            }).save();
        },
        accept: function(bookingId, requestorName) {
            var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), msg = String.format(L("widget_msg_booking_accept_system"), resourceOwner.get("alias")), self = this;
            return Alloy.createModel("ride_booking").accept(userId, bookingId, msg).then(function() {
                require("services/notification_service").updateCache(bookingId, "rideBookingRequests");
                eventDispatcher.emit(self.BOOKING_UPDATED);
                return Q.fcall(function() {
                    return true;
                });
            });
        },
        decline: function(bookingId, requestorName, declineAnimationTime) {
            declineAnimationTime = declineAnimationTime || 0;
            var resourceOwner = require("services/resource_owner_service").singleton(), userId = resourceOwner.get("userId"), msg = String.format(L("widget_msg_booking_decline_system"), resourceOwner.get("alias")), self = this;
            return Alloy.createModel("ride_booking").decline(userId, bookingId, msg).then(function() {
                require("services/notification_service").updateCache(bookingId, "rideBookingRequests");
                setTimeout(function() {
                    eventDispatcher.emit(self.BOOKING_UPDATED);
                }, declineAnimationTime);
                return Q();
            });
        },
        getRelevantBookings: function(data) {
            if (!data) return;
            var bookings = data.bookings || {}, myUserId = require("services/resource_owner_service").singleton().get("userId"), self = this;
            if (!bookings.length) return;
            var validBookings = _.filter(bookings, function(booking) {
                return booking.sourceTrip;
            });
            if (!validBookings.length) return;
            validBookings = _.groupBy(validBookings, function(booking) {
                var trip = booking.sourceTrip.ownerUid === myUserId ? booking.trip.baseTrip : booking.sourceTrip.baseTrip;
                return self.generateGroupingKeyForTrip(trip);
            });
            if (!validBookings) return;
            var updatedValidBookingRequests = [];
            for (var groupingKey in validBookings) {
                var group = validBookings[groupingKey];
                var relevantBooking = _.reduce(group, _compareBooking);
                updatedValidBookingRequests.push(relevantBooking);
            }
            return updatedValidBookingRequests;
        },
        generateGroupingKeyForTrip: function(trip) {
            if (!trip || !trip.schedule) {
                LOGGER.error("[bestMatches.js] - Trying to generate key of an empty trip");
                return;
            }
            return trip.id + "-" + trip.schedule.baseEffectiveTime;
        },
        on: function(event, callback) {
            eventDispatcher.on(event, callback);
        },
        emit: function(event) {
            eventDispatcher.emit(event);
        },
        removeAllListeners: function() {
            eventDispatcher.removeAllListeners();
        }
    };
};

BookingService.$isRevealingModule = true;

module.exports = BookingService;