"use strict";

var Q = require("vendor/q"), i18n = require("i18n/i18n"), resourceService = require("services/resource_owner_service").singleton(), Avatar = require("ui/common/Avatar"), URI = require("ui/link");

var MESSAGE_TYPE = "USER_TEXT", USER_POKE_TYPE = "USER_POKE", BOOKING_REQUEST_TYPE = "BOOKING_REQUESTED", BOOKING_ACCEPTED_TYPE = "BOOKING_ACCEPTED", BOOKING_CANCELLED_TYPE = "BOOKING_CANCELLED";

var Internal = {};

Internal.formatDate = function(timestamp) {
    var now = moment(), time = moment(timestamp), diff = Math.abs(time.diff(now, "days"));
    return diff ? i18n.formatDateTime(moment(timestamp)) : i18n.formatTime(moment(timestamp));
};

Internal.isOwnMessage = function(message) {
    var userId = resourceService.get("userId");
    return message.sendingUser === userId;
};

Internal.getMessageIcon = function(message) {
    if (message.messageType === USER_POKE_TYPE) return "x";
    return "";
};

var ChatMessageListItem = function(message, options) {
    message = message || {};
    options = options || {};
    options.showAvatar = options.showAvatar;
    options.showDate = options.showDate;
    options.isGroupChat = options.isGroupChat;
    options.previous = options.previous;
    var defaultAvatar = Avatar.getDefaultPhoto(message.user), avatarImage = Avatar.getLocalPhoto(message.user, {
        circular: true
    });
    this.template = "messageTemplate";
    this.isOwnMessage = Internal.isOwnMessage(message);
    this.messageType = message.messageType;
    this.sendingUser = message.sendingUser;
    this.user = message.user;
    this.timestamp = message.timestamp;
    this.avatar = {
        defaultImage: avatarImage || defaultAvatar,
        image: avatarImage || defaultAvatar
    };
    this.isAvatarLoaded = !!avatarImage;
    this.message = {
        value: "",
        text: ""
    };
    this.link = {
        href: "",
        text: ""
    };
    this.messageIcon = {
        text: Internal.getMessageIcon(message)
    };
    this.date = {
        text: Internal.formatDate(message.timestamp)
    };
    this.avatarStyle = {};
    this.updateMessage(message.body);
    this.updateStyles(options);
};

ChatMessageListItem.prototype.loadAvatar = function(offline) {
    if (this.isAvatarLoaded) return Q();
    var self = this;
    var updateAvatar = function(imagePath) {
        if (!imagePath) return Q();
        self.isAvatarLoaded = true;
        self.avatar = {
            defaultImage: imagePath,
            image: imagePath
        };
        return Q(self);
    };
    var imagePath = Avatar.getLocalPhoto(this.user, {
        circular: true
    });
    if (imagePath) {
        updateAvatar(imagePath);
        return Q();
    }
    if (!offline) {
        var dfd = Q.defer();
        Avatar.getUserPhoto(this.user, false, {
            circular: true
        }).then(updateAvatar).then(function() {
            dfd.resolve(self);
        });
        return dfd.promise;
    }
    return Q();
};

ChatMessageListItem.prototype.updateMessage = function(str) {
    str = str || "";
    this.messageType === USER_POKE_TYPE && (str = L(this.isOwnMessage ? "widget_messenger_you_poked" : "widget_messenger_poked_you"));
    this.link = URI.getLink(str);
    this.message.value = URI.convertHrefToText(str).trim();
    if (this.link.href) {
        this.message.value = this.message.value + "\n\n" + L("widget_messenger_link_mark") + " " + (this.link.text || this.link.href);
        this.message.touchEnabled = false;
    }
    this.message.text = this.message.value;
};

ChatMessageListItem.prototype.updateStyles = function(options) {
    options = options || {};
    options = _.extend({}, options, this.defaultsStyle(options.previous));
    if (!options.showAvatar) {
        this.avatarStyle.height = "0";
        this.avatarStyle.width = "0";
    }
    if (!options.showDate) {
        this.date.height = "0";
        this.date.text = "";
    }
    if (this.messageIcon.text) {
        _.extend(this.messageIcon, {
            width: "16dp",
            height: "16dp"
        });
        _.extend(this.message, {
            left: "25dp"
        });
    } else _.extend(this.messageIcon, {
        left: 0,
        width: "0dp",
        height: "0dp"
    });
};

ChatMessageListItem.prototype.defaultsStyle = function(previous) {
    var options = {};
    var hoursdiff = this.getHoursDifference(previous);
    options.showAvatar = options.showDate = this.haveSameOrigin(previous) ? hoursdiff > 1 ? true : false : true;
    return options;
};

ChatMessageListItem.prototype.haveSameOrigin = function(message) {
    if (!message) return false;
    return this.sendingUser == message.sendingUser;
};

ChatMessageListItem.prototype.getHoursDifference = function(message) {
    message = message || {};
    var d1 = moment(this.timestamp), d2 = moment(message.timestamp);
    return d1.diff(d2, "hours");
};

module.exports = ChatMessageListItem;